<?php
/*
 * $Horde: whups/admin.php,v 1.7 2002/02/28 04:08:51 chuck Exp $
 *
 * Copyright 2002 Chuck Hagenbuch <chuck@horde.org>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/bsdl.php.
 */

define('WHUPS_BASE', dirname(__FILE__));
require_once WHUPS_BASE . '/lib/base.php';
require_once WHUPS_BASE . '/lib/Admin.php';

require WHUPS_TEMPLATES . '/common.inc';
require WHUPS_TEMPLATES . '/page.inc';
require WHUPS_TEMPLATES . '/headerbar.inc';
require WHUPS_TEMPLATES . '/formrenderer.inc';
require WHUPS_TEMPLATES . '/actions.inc';
require WHUPS_TEMPLATES . '/actionresult.inc';

if (!@is_array($conf['auth']['admins'])) {
    exit('there are no administrators. administration is disabled.');
}

if (!Auth::getAuth() || !in_array(Auth::getAuth(), $conf['auth']['admins'])) {
    exit('forbidden.');
}

useForms(array('AddModuleForm',
               'EditModuleStep1Form',
               'EditModuleStep2Form',
               'DeleteModuleForm',
               'AddUserForm',
               'EditUserStep1Form',
               'AddTypeStep1Form',
               'EditTypeStep1Form',
               'EditTypeStep2Form',
               'DeleteTypeForm',
               'AddStateForm',
               'EditStateStep1Form',
               'EditStateStep2Form',
               'DeleteStateForm',
               'AddPriorityForm',
               'EditPriorityStep1Form',
               'EditPriorityStep2Form',
               'DeletePriorityForm',
               'AddSubjectForm',
               'EditSubjectStep1Form',
               'EditSubjectStep2Form',
               'DeleteSubjectForm'));

// set up the page config vars
$showExtraForm = null;

// admin actions
$AACTIONS = new Actions();
$AACTIONS->addAction(_("Edit Modules"), 'admin.php', 'module');
$AACTIONS->addAction(_("Edit Types"), 'admin.php', 'type');
$AACTIONS->addAction(_("Edit Subjects"), 'admin.php', 'subject');

$PAGE = new Page();
$HEADERBAR = new HeaderBar();
$RENDERER = new FormRenderer();
$ACTIONRESULT = new ActionResult();

// setup vars with all relevant info
$vars = &getDefaultVars();

// start the page
$PAGE->open(_("Administration"));
$HEADERBAR->header($sections, 'admin');
$HEADERBAR->actions($sections, 'admin');

// whatnow?
if ($vars->isVarSet('formname')) {
    switch ($vars->getVar('formname')) {
    case 'addtypestep1form':
        $whups_admin_form = 'type';
        $form1 = &getAddTypeStep1Form($vars);
        $form1->validate($vars);

        if ($form1->isValid()) {
            // First, add the type
            $tid = $backend->addType($vars->getVar('name'), $vars->getVar('description'));
            if (PEAR::isError($tid)) {
                Horde::fatal($tid, __FILE__, __LINE__);
            }
            
            $type = array('type' => $tid);
            $vars = new WhupsVars($type, $vars);
            
            $form2 = &getEditTypeStep2Form($vars);
            $form2->title = 'addtypestep2form';
            $form2->open($RENDERER, $vars, 'admin.php', 'post');

            // render the stage 1 form readonly
            $form1->preserve($vars);
            $RENDERER->beginInactive(_("Add Type - Stage 1"));
            $RENDERER->renderFormInactive($form1, $vars);
            $RENDERER->end();

            // render the second stage form
            $RENDERER->beginActive(_("Add Type - Stage 2"));
            $RENDERER->renderFormActive($form2, $vars);
            $RENDERER->submit();
            $RENDERER->end();

            $form2->close($RENDERER);
        } else {
            $form1->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(_("Add Type - Stage 1"));
            $RENDERER->renderFormActive($form1, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form1->close($RENDERER);
        }
        break;
        
    case 'addtypestep2form':
        $whups_admin_form = 'type';
        $form1 = &getAddTypeStep1Form($vars);
        $form2 = &getEditTypeStep2Form($vars);
        $form2->title = 'addtypestep2form';
        break;

    case 'edittypestep1form':
        $whups_admin_form = 'type';
        $form1 = &getEditTypeStep1Form($vars);
        $form1->validate($vars);

        if ($form1->isValid()) {
            switch ($vars->getVar('submit')) {
            case _("Edit Type"):
                $form2 = &getEditTypeStep2Form($vars);
                $form2->open($RENDERER, $vars, 'admin.php', 'post');

                // render the second stage form
                $RENDERER->beginActive(_("Edit Type - Stage 2"));
                $RENDERER->renderFormActive($form2, $vars);
                $RENDERER->submit();
                $RENDERER->end();

                $form2->close($RENDERER);
                break;
                
            case _("Delete Type"):
                $form2 = &getDeleteTypeForm($vars);
                $form2->open($RENDERER, $vars, 'admin.php', 'post');
                
                // render the deletion form
                $RENDERER->beginActive(_("Delete Type - Confirmation"));
                $RENDERER->renderFormActive($form2, $vars);
                $RENDERER->submit();
                $RENDERER->end();

                $form2->close($RENDERER);
                break;
            }
        } else {
            $form1->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(_("Edit Type"));
            $RENDERER->renderFormActive($form1, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form1->close($RENDERER);
        }
        break;

    case 'edittypestep2form':
        $whups_admin_form = 'type';
        $form1 = &getEditTypeStep2Form($vars);
        $form1->validate($vars);
        
        if ($form1->isValid()) {
            $result = $backend->updateType($vars->getVar('type'), $vars->getVar('name'),
                                           $vars->getVar('description'));
            if (PEAR::isError($result)) {
                $ACTIONRESULT->result(_("Edit Type"), 0, sprintf(_("There was an error modifying the type: %s."), $result->getMessage()));
            } else {
                $ACTIONRESULT->result(_("Edit Type"), 1, sprintf(_("The type '%s' has been modified."), $vars->getVar('name')));
                $form1->open($RENDERER, $vars, 'admin.php', 'post');
                $RENDERER->beginInactive(_("Modified Type"));
                $RENDERER->renderFormInactive($form1, $vars);
                $RENDERER->end();
                $form1->close($RENDERER);
            }
        } else {
            $form1->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(_("Edit Type"));
            $RENDERER->renderFormActive($form1, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form1->close($RENDERER);
        }
        break;

    case 'deletetypeform':
        $whups_admin_form = 'type';
        $form = &getDeleteTypeForm($vars);
        $form->validate($vars);
        
        if ($form->isValid()) {
            if ($vars->getVar('yesno') == 1) {
                $result = $backend->deleteType($vars->getVar('type'));
                if (!PEAR::isError($result)) {
                    $ACTIONRESULT->result(_("Delete Type"), 1, _("The type has been deleted."));
                } else {
                    $ACTIONRESULT->result(_("Delete Type"), 0, sprintf(_("There was an error deleting the type: %s."), $result->getMessage()));
                }
            } else {
                $ACTIONRESULT->result(_("Delete Type"), 1, _("The type was not deleted."));
            }
        } else {
            $form->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(_("Delete Type - Confirmation"));
            $RENDERER->renderFormActive($form, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form->close($RENDERER);
        }
        break;
        
    case 'addmoduleform':
        $whups_admin_form = 'module';
        $form = &getAddModuleForm($vars);
        $form->validate($vars);

        if ($form->isValid()) {
            $result = $backend->addModule($vars->getVar('name'), $vars->getVar('description'));
            if (!PEAR::isError($result)) {
                $ACTIONRESULT->result(_("Add Module"), 1, sprintf(_("The module '%s' has been created."), $vars->getVar('name')));
            } else {
                $ACTIONRESULT->result(_("Add Module"), 0, sprintf(_("There was an error creating the module: %s."), $result->getMessage()));
            }
        } else {
            $form->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(_("Add Module"));
            $RENDERER->renderFormActive($form, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form->close($RENDERER);
        }
        break;
        
    case 'editmodulestep1form':
        $whups_admin_form = 'module';
        $form1 = &getEditModuleStep1Form($vars);
        $form1->validate($vars);

        if ($form1->isValid()) {
            switch ($vars->getVar('submit')) {
            case _("Edit Module"):
                $form2 = &getEditModuleStep2Form($vars);
                $form2->open($RENDERER, $vars, 'admin.php', 'post');

                // render the second stage form
                $RENDERER->beginActive(_("Edit Module - Stage 2"));
                $RENDERER->renderFormActive($form2, $vars);
                $RENDERER->submit();
                $RENDERER->end();

                $form2->close($RENDERER);
                break;
                
            case _("Delete Module"):
                $form2 = &getDeleteModuleForm($vars);
                $form2->open($RENDERER, $vars, 'admin.php', 'post');
                
                // render the deletion form
                $RENDERER->beginActive(_("Delete Module - Confirmation"));
                $RENDERER->renderFormActive($form2, $vars);
                $RENDERER->submit();
                $RENDERER->end();

                $form2->close($RENDERER);
                break;
            }
        } else {
            $form1->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(_("Edit Module"));
            $RENDERER->renderFormActive($form1, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form1->close($RENDERER);
        }
        break;
        
    case 'editmodulestep2form':
        $whups_admin_form = 'module';
        $form1 = &getEditModuleStep2Form($vars);
        $form1->validate($vars);

        if ($form1->isValid()) {
            // update everything.
            $result = $backend->updateModule($vars->getVar('module'), $vars->getVar('name'),
                                             $vars->getVar('description'), $vars->getVar('types'),
                                             $vars->getVar('subjectlist'), $vars->getVar('subjects'));
            if (!PEAR::isError($result)) {
                $ACTIONRESULT->result(_("Edit Module"), 1, _("The module has been modified."));
                $form1->open($RENDERER, $vars, 'admin.php', 'post');
                $RENDERER->beginInactive(_("Modified Module"));
                $RENDERER->renderFormInactive($form1, $vars);
                $RENDERER->end();
                $form1->close($RENDERER);
            } else {
                $ACTIONRESULT->result(_("Edit Module"), 0, sprintf(_("There was an error editing the module: %s."), $result->getMessage()));
            }
        } else {
            $form1->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(_("Edit Module - Stage 2"));
            $RENDERER->renderFormActive($form1, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form1->close($RENDERER);
        }

        break;

    case 'deletemoduleform':
        $whups_admin_form = 'module';
        $form = &getDeleteModuleForm($vars);
        $form->validate($vars);
        
        if ($form->isValid()) {
            if ($vars->getVar('yesno') == 1) {
                $result = $backend->deleteModule($vars->getVar('module'));
                if (!PEAR::isError($result)) {
                    $ACTIONRESULT->result(_("Delete Module"), 1, _("The module has been deleted."));
                } else {
                    $ACTIONRESULT->result(_("Delete Module"), 0, sprintf(_("There was an error deleting the module: %s."), $result->getMessage()));
                }
            } else {
                $ACTIONRESULT->result(_("Delete Module"), 1, _("The module was not deleted."));
            }
        } else {
            $form->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(_("Delete Module - Confirmation"));
            $RENDERER->renderFormActive($form, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form->close($RENDERER);
        }
        break;

    case 'addstateform':
        $whups_admin_form = 'type';
        $form = &getAddStateForm($vars);
        $form->validate($vars);
        $typename = $backend->getType($vars->getVar('type'));
        $typename = $typename['name'];

        if ($form->isValid()) {
            $result = $backend->addState($vars->getVar('type'), $vars->getVar('name'), $vars->getVar('description'), $vars->getVar('category'));
            if (!PEAR::isError($result)) {
                $ACTIONRESULT->result(_("Add State"), 1, sprintf(_("The state '%s' has been added to %s."), $vars->getVar('name'), $typename));
            } else {
                $ACTIONRESULT->result(_("Add State"), 0, sprintf(_("There was an error creating the state: %s."), $result->getMessage()));
            }

            $vars = array('type' => $vars->getVar('type'));
            $vars2 = array();
            $vars = new WhupsVars($vars, $vars2);
            $form1 = &getEditStateStep1Form($vars);
            $form1->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(sprintf(_("Edit States for '%s' - Stage 1"), $typename));
            $RENDERER->renderFormActive($form1, $vars);
            $RENDERER->submit(_("Edit State"));
            $RENDERER->submit(_("Delete State"));
            $RENDERER->end();
            $form1->close($RENDERER);

            $form2 = &getAddStateForm($vars);
            $form2->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(sprintf(_("Add State for '%s'"), $typename));
            $RENDERER->renderFormActive($form2, $vars);
            $RENDERER->submit(_("Add State"));
            $RENDERER->end();
            $form2->close($RENDERER);
        } else {
            $form->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(sprintf(_("Add State for '%s'"), $typename));
            $RENDERER->renderFormActive($form, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form->close($RENDERER);
        }
        break;
        
    case 'editstatestep1form':
        $whups_admin_form = 'type';
        $form1 = &getEditStateStep1Form($vars);
        $typename = $backend->getType($vars->getVar('type'));
        $typename = $typename['name'];
        if (!$vars->getVar('submit')) {
            $form1->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(sprintf(_("Edit States for '%s' - Stage 1"), $typename));
            $RENDERER->renderFormActive($form1, $vars);
            $RENDERER->submit(_("Edit State"));
            $RENDERER->submit(_("Delete State"));
            $RENDERER->end();
            $form1->close($RENDERER);
            
            $form2 = &getAddStateForm($vars);
            $form2->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(sprintf(_("Add State for '%s'"), $typename));
            $RENDERER->renderFormActive($form2, $vars);
            $RENDERER->submit(_("Add State"));
            $RENDERER->end();
            $form2->close($RENDERER);
        } else {
            $form1->validate($vars);

            if ($form1->isValid()) {
                switch ($vars->getVar('submit')) {
                case _("Edit State"):
                    $form2 = &getEditStateStep2Form($vars);
                    $form2->open($RENDERER, $vars, 'admin.php', 'post');

                    // render the second stage form
                    $RENDERER->beginActive(_("Edit State - Stage 2"));
                    $RENDERER->renderFormActive($form2, $vars);
                    $RENDERER->submit();
                    $RENDERER->end();

                    $form2->close($RENDERER);
                    break;
                
                case _("Delete State"):
                    $form2 = &getDeleteStateForm($vars);
                    $form2->open($RENDERER, $vars, 'admin.php', 'post');

                    // render the deletion form
                    $RENDERER->beginActive(_("Delete State - Confirmation"));
                    $RENDERER->renderFormActive($form2, $vars);
                    $RENDERER->submit();
                    $RENDERER->end();

                    $form2->close($RENDERER);
                    break;
                }
            } else {
                $form1->open($RENDERER, $vars, 'admin.php', 'post');
                $RENDERER->beginActive(_("Edit States"));
                $RENDERER->renderFormActive($form1, $vars);
                $RENDERER->submit();
                $RENDERER->end();
                $form1->close($RENDERER);
            }
        }
        break;

    case 'editstatestep2form':
        $whups_admin_form = 'type';
        $form = &getEditStateStep2Form($vars);
        $form->validate($vars);

        if ($form->isValid()) {
            // update everything.
            $result = $backend->updateState($vars->getVar('state'), $vars->getVar('name'),
                                            $vars->getVar('description'), $vars->getVar('category'));
            if (!PEAR::isError($result)) {
                $ACTIONRESULT->result(_("Edit State"), 1, _("The state has been modified."));
                $form->open($RENDERER, $vars, 'admin.php', 'post');
                $RENDERER->beginInactive(_("Modified State"));
                $RENDERER->renderFormInactive($form, $vars);
                $RENDERER->end();
                $form->close($RENDERER);
            } else {
                $ACTIONRESULT->result(_("Edit State"), 0, sprintf(_("There was an error editing the state: %s."), $result->getMessage()));
            }

            $vars = array('type' => $vars->getVar('type'));
            $vars2 = array();
            $vars = new WhupsVars($vars, $vars2);
            $typename = $backend->getType($vars->getVar('type'));
            $typename = $typename['name'];

            $form1 = &getEditStateStep1Form($vars);
            $form1->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(sprintf(_("Edit States for '%s' - Stage 1"), $typename));
            $RENDERER->renderFormActive($form1, $vars);
            $RENDERER->submit(_("Edit State"));
            $RENDERER->submit(_("Delete State"));
            $RENDERER->end();
            $form1->close($RENDERER);

            $form2 = &getAddStateForm($vars);
            $form2->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(sprintf(_("Add State for '%s'"), $typename));
            $RENDERER->renderFormActive($form2, $vars);
            $RENDERER->submit(_("Add State"));
            $RENDERER->end();
            $form2->close($RENDERER);
        } else {
            $form1->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(_("Edit State - Stage 2"));
            $RENDERER->renderFormActive($form1, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form1->close($RENDERER);
        }
        break;

    case 'deletestateform':
        $whups_admin_form = 'type';
        $form = &getDeleteStateForm($vars);
        $form->validate($vars);

        if ($form->isValid()) {
            if ($vars->getVar('yesno') == 1) {
                $result = $backend->deleteState($vars->getVar('state'));
                if (!PEAR::isError($result)) {
                    $ACTIONRESULT->result(_("Delete State"), 1, _("The state has been deleted."));
                } else {
                    $ACTIONRESULT->result(_("Delete State"), 0, sprintf(_("There was an error deleting the state: %s."), $result->getMessage()));
                }
            } else {
                $ACTIONRESULT->result(_("Delete State"), 1, _("The state was not deleted."));
            }

            $typename = $backend->getType($vars->getVar('type'));
            $typename = $typename['name'];
            $form1 = &getEditStateStep1Form($vars);
            $form1->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(sprintf(_("Edit States for '%s' - Stage 1"), $typename));
            $RENDERER->renderFormActive($form1, $vars);
            $RENDERER->submit(_("Edit State"));
            $RENDERER->submit(_("Delete State"));
            $RENDERER->end();
            $form1->close($RENDERER);

            $form2 = &getAddStateForm($vars);
            $form2->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(sprintf(_("Add State for '%s'"), $typename));
            $RENDERER->renderFormActive($form2, $vars);
            $RENDERER->submit(_("Add State"));
            $RENDERER->end();
            $form2->close($RENDERER);
        } else {
            $form->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(_("Delete State - Confirmation"));
            $RENDERER->renderFormActive($form, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form->close($RENDERER);
        }
        break;

    case 'addpriorityform':
        $whups_admin_form = 'type';
        $form = &getAddPriorityForm($vars);
        $form->validate($vars);
        $typename = $backend->getType($vars->getVar('type'));
        $typename = $typename['name'];

        if ($form->isValid()) {
            $result = $backend->addPriority($vars->getVar('type'), $vars->getVar('name'), $vars->getVar('description'));
            if (!PEAR::isError($result)) {
                $ACTIONRESULT->result(_("Add Priority"), 1, sprintf(_("The priority '%s' has been added to %s."), $vars->getVar('name'), $typename));
            } else {
                $ACTIONRESULT->result(_("Add Priority"), 0, sprintf(_("There was an error creating the priority: %s."), $result->getMessage()));
            }

            $vars = array('type' => $vars->getVar('type'));
            $vars2 = array();
            $vars = new WhupsVars($vars, $vars2);
            $form1 = &getEditPriorityStep1Form($vars);
            $form1->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(sprintf(_("Edit Priorities for '%s' - Stage 1"), $typename));
            $RENDERER->renderFormActive($form1, $vars);
            $RENDERER->submit(_("Edit Priority"));
            $RENDERER->submit(_("Delete Priority"));
            $RENDERER->end();
            $form1->close($RENDERER);

            $form2 = &getAddPriorityForm($vars);
            $form2->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(sprintf(_("Add Priority for '%s'"), $typename));
            $RENDERER->renderFormActive($form2, $vars);
            $RENDERER->submit(_("Add Priority"));
            $RENDERER->end();
            $form2->close($RENDERER);
        } else {
            $form->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(sprintf(_("Add Priority for '%s'"), $typename));
            $RENDERER->renderFormActive($form, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form->close($RENDERER);
        }
        break;
        
    case 'editprioritystep1form':
        $whups_admin_form = 'type';
        $form1 = &getEditPriorityStep1Form($vars);
        $typename = $backend->getType($vars->getVar('type'));
        $typename = $typename['name'];
        if (!$vars->getVar('submit')) {
            $form1->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(sprintf(_("Edit Priorities for '%s' - Stage 1"), $typename));
            $RENDERER->renderFormActive($form1, $vars);
            $RENDERER->submit(_("Edit Priority"));
            $RENDERER->submit(_("Delete Priority"));
            $RENDERER->end();
            $form1->close($RENDERER);
            
            $form2 = &getAddPriorityForm($vars);
            $form2->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(sprintf(_("Add Priority for '%s'"), $typename));
            $RENDERER->renderFormActive($form2, $vars);
            $RENDERER->submit(_("Add Priority"));
            $RENDERER->end();
            $form2->close($RENDERER);
        } else {
            $form1->validate($vars);

            if ($form1->isValid()) {
                switch ($vars->getVar('submit')) {
                case _("Edit Priority"):
                    $form2 = &getEditPriorityStep2Form($vars);
                    $form2->open($RENDERER, $vars, 'admin.php', 'post');

                    // render the second stage form
                    $RENDERER->beginActive(_("Edit Priority - Stage 2"));
                    $RENDERER->renderFormActive($form2, $vars);
                    $RENDERER->submit();
                    $RENDERER->end();

                    $form2->close($RENDERER);
                    break;
                
                case _("Delete Priority"):
                    $form2 = &getDeletePriorityForm($vars);
                    $form2->open($RENDERER, $vars, 'admin.php', 'post');

                    // render the deletion form
                    $RENDERER->beginActive(_("Delete Priority - Confirmation"));
                    $RENDERER->renderFormActive($form2, $vars);
                    $RENDERER->submit();
                    $RENDERER->end();

                    $form2->close($RENDERER);
                    break;
                }
            } else {
                $form1->open($RENDERER, $vars, 'admin.php', 'post');
                $RENDERER->beginActive(_("Edit Priorities"));
                $RENDERER->renderFormActive($form1, $vars);
                $RENDERER->submit();
                $RENDERER->end();
                $form1->close($RENDERER);
            }
        }
        break;

    case 'editprioritystep2form':
        $whups_admin_form = 'type';
        $form = &getEditPriorityStep2Form($vars);
        $form->validate($vars);

        if ($form->isValid()) {
            // update everything.
            $result = $backend->updatePriority($vars->getVar('priority'), $vars->getVar('name'),
                                            $vars->getVar('description'));
            if (!PEAR::isError($result)) {
                $ACTIONRESULT->result(_("Edit Priority"), 1, _("The priority has been modified."));
                $form->open($RENDERER, $vars, 'admin.php', 'post');
                $RENDERER->beginInactive(_("Modified Priority"));
                $RENDERER->renderFormInactive($form, $vars);
                $RENDERER->end();
                $form->close($RENDERER);
            } else {
                $ACTIONRESULT->result(_("Edit Priority"), 0, sprintf(_("There was an error editing the priority: %s."), $result->getMessage()));
            }

            $vars = array('type' => $vars->getVar('type'));
            $vars2 = array();
            $vars = new WhupsVars($vars, $vars2);
            $typename = $backend->getType($vars->getVar('type'));
            $typename = $typename['name'];

            $form1 = &getEditPriorityStep1Form($vars);
            $form1->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(sprintf(_("Edit Priorities for '%s' - Stage 1"), $typename));
            $RENDERER->renderFormActive($form1, $vars);
            $RENDERER->submit(_("Edit Priority"));
            $RENDERER->submit(_("Delete Priority"));
            $RENDERER->end();
            $form1->close($RENDERER);

            $form2 = &getAddPriorityForm($vars);
            $form2->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(sprintf(_("Add Priority for '%s'"), $typename));
            $RENDERER->renderFormActive($form2, $vars);
            $RENDERER->submit(_("Add Priority"));
            $RENDERER->end();
            $form2->close($RENDERER);
        } else {
            $form1->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(_("Edit Priority - Stage 2"));
            $RENDERER->renderFormActive($form1, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form1->close($RENDERER);
        }
        break;

    case 'deletepriorityform':
        $whups_admin_form = 'type';
        $form = &getDeletePriorityForm($vars);
        $form->validate($vars);

        if ($form->isValid()) {
            if ($vars->getVar('yesno') == 1) {
                $result = $backend->deletePriority($vars->getVar('priority'));
                if (!PEAR::isError($result)) {
                    $ACTIONRESULT->result(_("Delete Priority"), 1, _("The priority has been deleted."));
                } else {
                    $ACTIONRESULT->result(_("Delete Priority"), 0, sprintf(_("There was an error deleting the priority: %s."), $result->getMessage()));
                }
            } else {
                $ACTIONRESULT->result(_("Delete Priority"), 1, _("The priority was not deleted."));
            }

            $typename = $backend->getType($vars->getVar('type'));
            $typename = $typename['name'];
            $form1 = &getEditPriorityStep1Form($vars);
            $form1->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(sprintf(_("Edit Priorities for '%s' - Stage 1"), $typename));
            $RENDERER->renderFormActive($form1, $vars);
            $RENDERER->submit(_("Edit Priority"));
            $RENDERER->submit(_("Delete Priority"));
            $RENDERER->end();
            $form1->close($RENDERER);

            $form2 = &getAddPriorityForm($vars);
            $form2->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(sprintf(_("Add Priority for '%s'"), $typename));
            $RENDERER->renderFormActive($form2, $vars);
            $RENDERER->submit(_("Add Priority"));
            $RENDERER->end();
            $form2->close($RENDERER);
        } else {
            $form->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(_("Delete Priority - Confirmation"));
            $RENDERER->renderFormActive($form, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form->close($RENDERER);
        }
        break;

    case 'adduserform':
        $whups_admin_form = 'module';
        $form = &getAddUserForm($vars);
        $form->validate($vars);

        if ($form->isValid()) {
            $info = $backend->getModule($vars->getVar('module'));
            $result = $backend->addModuleUser($vars->getVar('module'), $vars->getVar('user'));
            if (!PEAR::isError($result)) {
                $ACTIONRESULT->result(_("Add Responsible User"), 1, sprintf(_("'%s' is now among those responsible for '%s'"), $vars->getVar('user'), $info['name']));
            } else {
                $ACTIONRESULT->result(_("Add Responsible User"), 0, sprintf(_("There was an error adding '%s' to the responsible list for '%s': %s"), $vars->getVar('user'), $info['name'], $result->getMessage()));
            }

            $vars = array('module' => $vars->getVar('module'));
            $vars2 = array();
            $vars = new WhupsVars($vars, $vars2);
            $form1 = &getEditUserStep1Form($vars);

            $form1->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(sprintf(_("Users responsible for '%s'"), $info['name']));
            $RENDERER->renderFormActive($form1, $vars);
            $RENDERER->submit(_("Remove User"));
            $RENDERER->end();
            $form1->close($RENDERER);

            $form->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(sprintf(_("Add a user to '%s'"), $info['name']));
            $RENDERER->renderFormActive($form, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form->close($RENDERER);
        } else {
            $form->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(_("Add Module"));
            $RENDERER->renderFormActive($form, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form->close($RENDERER);
        }
        break;

    case 'edituserform':
        $whups_admin_form = 'module';

        $info = $backend->getModule($vars->getVar('module'));

        $form1 = &getEditUserStep1Form($vars);
        $form2 = &getAddUserForm($vars);

        $form1->open($RENDERER, $vars, 'admin.php', 'post');
        $RENDERER->beginActive(sprintf(_("Users responsible for '%s'"), $info['name']));
        $RENDERER->renderFormActive($form1, $vars);
        $RENDERER->submit(_("Remove User"));
        $RENDERER->end();
        $form1->close($RENDERER);

        $form2->open($RENDERER, $vars, 'admin.php', 'post');
        $RENDERER->beginActive(sprintf(_("Add a user to '%s'"), $info['name']));
        $RENDERER->renderFormActive($form2, $vars);
        $RENDERER->submit();
        $RENDERER->end();
        $form2->close($RENDERER);

        break;

    case 'edituserstep1form':
        $whups_admin_form = 'module';
        $form = &getEditUserStep1Form($vars);
        $form->validate($vars);

        if ($form->isValid()) {
            $info = $backend->getModule($vars->getVar('module'));
            $result = $backend->removeModuleUser($vars->getVar('module'), $vars->getVar('user'));
            if (!PEAR::isError($result)) {
                $ACTIONRESULT->result(_("Remove Responsible User"), 1, sprintf(_("'%s' is no longer among those responsible for '%s'"), $vars->getVar('user'), $info['name']));
            } else {
                $ACTIONRESULT->result(_("Remove Responsible User"), 0, sprintf(_("There was an error removing '%s' from the responsible list for '%s': %s"), $vars->getVar('user'), $info['name'], $result->getMessage()));
            }

            $vars = array('module' => $vars->getVar('module'));
            $vars2 = array();
            $vars = new WhupsVars($vars, $vars2);
            $form->EditUserStep1Form($vars);
            $form1 = &getAddUserForm($vars);

            $form->open($RENDERER, $vars, 'admin.php', 'get');
            $RENDERER->beginActive(sprintf(_("Users responsible for '%s'"), $info['name']));
            $RENDERER->renderFormActive($form, $vars);
            $RENDERER->submit(_("Remove User"));
            $RENDERER->end();
            $form->close($RENDERER);

            $form1->open($RENDERER, $vars, 'admin.php', 'get');
            $RENDERER->beginActive(sprintf(_("Add a user to '%s'"), $info['name']));
            $RENDERER->renderFormActive($form1, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form1->close($RENDERER);
        } else {
            $form->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(_("Add Module"));
            $RENDERER->renderFormActive($form, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form->close($RENDERER);
        }
        break;

    case 'addsubjectform':
        $whups_admin_form = 'subject';
        $form = &getAddSubjectForm($vars);
        $form->validate($vars);

        if ($form->isValid()) {
            $result = $backend->addSubject($vars->getVar('name'));
            if (!PEAR::isError($result)) {
                $ACTIONRESULT->result(_("Add Subject"), 1, sprintf(_("The subject '%s' has been added."), $vars->getVar('name')));
            } else {
                $ACTIONRESULT->result(_("Add Subject"), 0, sprintf(_("There was an error adding the subject: %s."), $result->getMessage()));
            }
        } else {
            $form->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(_("Add Subject"));
            $RENDERER->renderFormActive($form, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form->close($RENDERER);
        }
        break;

    case 'editsubjectstep1form':
        $whups_admin_form = 'subject';
        $form1 = &getEditSubjectStep1Form($vars);
        $form1->validate($vars);

        if ($form1->isValid()) {
            switch ($vars->getVar('submit')) {
            case _("Edit Subject"):
                $form2 = &getEditSubjectStep2Form($vars);
                $form2->open($RENDERER, $vars, 'admin.php', 'post');

                // render the second stage form
                $RENDERER->beginActive(_("Edit Subject - Stage 2"));
                $RENDERER->renderFormActive($form2, $vars);
                $RENDERER->submit();
                $RENDERER->end();

                $form2->close($RENDERER);
                break;

            case _("Delete Subject"):
                $form2 = &getDeleteSubjectForm($vars);
                $form2->open($RENDERER, $vars, 'admin.php', 'post');

                // render the deletion form
                $RENDERER->beginActive(_("Delete Subject - Confirmation"));
                $RENDERER->renderFormActive($form2, $vars);
                $RENDERER->submit();
                $RENDERER->end();

                $form2->close($RENDERER);
                break;
            }
        } else {
            $form1->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(_("Edit Subject"));
            $RENDERER->renderFormActive($form1, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form1->close($RENDERER);
        }
        break;
        
    case 'editsubjectstep2form':
        $whups_admin_form = 'subject';
        $form1 = &getEditSubjectStep2Form($vars);
        $form1->validate($vars);

        if ($form1->isValid()) {
            // update everything.
            $result = $backend->updateSubject($vars->getVar('subject'), $vars->getVar('name'),
                                             $vars->getVar('description'), $vars->getVar('types'));
            if (!PEAR::isError($result)) {
                $ACTIONRESULT->result(_("Edit Subject"), 1, _("The subject has been modified."));
                $form1->open($RENDERER, $vars, 'admin.php', 'post');
                $RENDERER->beginInactive(_("Modified Subject"));
                $RENDERER->renderFormInactive($form1, $vars);
                $RENDERER->end();
                $form1->close($RENDERER);
            } else {
                $ACTIONRESULT->result(_("Edit Subject"), 0, sprintf(_("There was an error editing the subject: %s."), $result->getMessage()));
            }
        } else {
            $form1->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(_("Edit Subject - Stage 2"));
            $RENDERER->renderFormActive($form1, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form1->close($RENDERER);
        }

        break;

    case 'deletesubjectform':
        $whups_admin_form = 'subject';
        $form = &getDeleteSubjectForm($vars);
        $form->validate($vars);
        
        if ($form->isValid()) {
            if ($vars->getVar('yesno') == 1) {
                $result = $backend->deleteSubject($vars->getVar('subject'));
                if (!PEAR::isError($result)) {
                    $ACTIONRESULT->result(_("Delete Subject"), 1, _("The subject has been deleted."));
                } else {
                    $ACTIONRESULT->result(_("Delete Subject"), 0, sprintf(_("There was an error deleting the subject: %s."), $result->getMessage()));
                }
            } else {
                $ACTIONRESULT->result(_("Delete Subject"), 1, _("The subject was not deleted."));
            }
        } else {
            $form->open($RENDERER, $vars, 'admin.php', 'post');
            $RENDERER->beginActive(_("Delete Subject - Confirmation"));
            $RENDERER->renderFormActive($form, $vars);
            $RENDERER->submit();
            $RENDERER->end();
            $form->close($RENDERER);
        }
        break;

    default:
        $whups_admin_form = 'module';
        break;

    }
} else {
    // check for actions from the useractions template class
    switch ($vars->getVar('action')) {
    case 'type':
        $whups_admin_form = 'type';
        $form1 = &getEditTypeStep1Form($vars); $edit1 = _("Edit Type"); $edit2 = _("Delete Type");
        $form2 = &getAddTypeStep1Form($vars); $add = _("Add Type - Step 1");
        break;

    case 'subject':
        $whups_admin_form = 'subject';
        $form1 = &getEditSubjectStep1Form($vars); $edit1 = _("Edit Subject"); $edit2 = _("Delete Subject");
        $form2 = &getAddSubjectForm($vars); $add = _("Add Subject");
        break;

    default:
    case 'module':
        $whups_admin_form = 'module';
        $form1 = &getEditModuleStep1Form($vars); $edit1 = _("Edit Module"); $edit2 = _("Delete Module");
        $form2 = &getAddModuleForm($vars); $add = _("Add Module");
        break;
    }

    $form1->open($RENDERER, $vars, 'admin.php', 'get');
    $RENDERER->beginActive($edit1);
    $RENDERER->renderFormActive($form1, $vars);
    $RENDERER->submit($edit1);
    $RENDERER->submit($edit2);
    $RENDERER->end();
    $form1->close($RENDERER);

    $form2->open($RENDERER, $vars, 'admin.php', 'get');
    $RENDERER->beginActive($add);
    $RENDERER->renderFormActive($form2, $vars);
    $RENDERER->submit($add);
    $RENDERER->end();
    $form2->close($RENDERER);
}

$AACTIONS->render($whups_admin_form);

$PAGE->close();

?>
