<?php
/*
 * $Horde: whups/lib/Admin.php,v 1.7 2002/02/28 04:08:52 chuck Exp $
 *
 * Copyright 2002 Chuck Hagenbuch <chuck@horde.org>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/bsdl.php.
 */

require_once WHUPS_BASE . '/lib/Forms.php';

class AddSubjectForm extends Form {

    function AddSubjectForm(&$vars)
    {
        $this->Form($vars, 'addsubjectform');

        $var1 = new Variable(_("Subject Name"), 'name', getTextType(), true);

        $this->addVariable($var1);
    }

}

class EditSubjectStep1Form extends Form {

    function EditSubjectStep1Form(&$vars)
    {
        global $backend;

        $this->Form($vars, 'editsubjectstep1form');

        $subjects = $backend->getSubjects();
        $subtype = new EnumType($subjects);

        $var1 = new Variable(_("Subject Name"), 'subject', $subtype, true);

        $this->addVariable($var1);
    }

}

class EditSubjectStep2Form extends Form {

    function EditSubjectStep2Form(&$vars)
    {
        global $backend;

        $this->Form($vars, 'editsubjectstep2form');

        $subject = $vars->getVar('subject');
        $info = $backend->getSubject($subject);

        $var1 = new Variable('', 'subject', getIntType(), true, true);
        $var2 = new Variable(_("Subject Name"), 'name', getTextType(), true);

        $var2->setDefault($info['name']);

        $this->addHidden($var1);
        $this->addVariable($var2);
    }

}

class DeleteSubjectForm extends Form {

    function DeleteSubjectForm(&$vars)
    {
        global $backend;
        
        $this->Form($vars, 'deletesubjectform');
        
        $subject = $vars->getVar('subject');
        $info = $backend->getSubject($subject);

        $yesnotype = new EnumType(array(0 => _("No"), 1 => _("Yes")));

        $var1 = new Variable('', 'subject', getIntType(), true, true);
        $var2 = new Variable(_("Subject Name"), 'name', getTextType(), false, true);
        $var3 = new Variable(_("Really delete this subject? This may cause data problems!!"), 'yesno', $yesnotype, true);

        $var2->setDefault($info['name']);

        $this->addHidden($var1);
        $this->addVariable($var2);
        $this->addVariable($var3);
    }

}

class AddModuleForm extends Form {

    function AddModuleForm(&$vars)
    {
        $this->Form($vars, 'addmoduleform');

        $var1 = new Variable(_("Module Name"), 'name', getTextType(), true);
        $var2 = new Variable(_("Module Description"), 'description', getTextType(), true);

        $this->addVariable($var1);
        $this->addVariable($var2);
    }
    
}

class EditModuleStep1Form extends Form {

    function EditModuleStep1Form(&$vars)
    {
        global $backend;

        $this->Form($vars, 'editmodulestep1form');

        $modules = $backend->getModules();
        $modtype = new EnumType($modules);

        $var1 = new Variable(_("Module Name"), 'module', $modtype, true);

        $this->addVariable($var1);
    }
    
}

class EditModuleStep2Form extends Form {

    function EditModuleStep2Form(&$vars)
    {
        global $backend;

        $this->Form($vars, 'editmodulestep2form');

        $module = $vars->getVar('module');
        $info = $backend->getModule($module);
        $types = $backend->getAllTypes();
        $settype = new SetType($types);
        $subjects = $backend->getSubjects();
        $subtype = new SetType($subjects);
        $users = $backend->getModuleUsers($module);
        $usertype = new SetType($users);
        $userlink = new LinkType(array('text' => _("Edit the users responsible for this module"), 'url' => Horde::applicationUrl('admin.php?formname=edituserform&module=' . $module)));
        $permslink = new LinkType(array('text' => _("Edit the permissions on this module"), 'url' => Horde::url($GLOBALS['registry']->getParam('webroot', 'horde') . '/admin/perms.php?actionID=edit&category=' . urlencode("whups_module_$module") . '&parent[0]=whups&parent[1]=whups_modules&autocreate=1')));

        $var1 = new Variable('', 'module', getIntType(), true, true);
        $var2 = new Variable(_("Module Name"), 'name', getTextType(), true);
        $var3 = new Variable(_("Module Description"), 'description', getTextType(), true);
        $var4 = new Variable(_("Types associated with this Module"), 'types', $settype, true);
        $var5 = new Variable(_("Restrict ticket subjects to a certain set"), 'subjectlist', getBooleanType(), true);
        $var6 = new Variable(_("Subjects associated with this Module"), 'subjects', $subtype, true);
        $var7 = new Variable(_("Users responsible for this Module"), 'users', $usertype, false, true);
        $var8 = new Variable('', 'link', $userlink, false, true);
        $var9 = new Variable('', 'link', $permslink, false, true);

        $var2->setDefault($info['name']);
        $var3->setDefault($info['description']);
        $var4->setDefault(array_keys($backend->getTypes($module)));
        $var5->setDefault($info['subjectlist']);
        $var6->setDefault(array_keys($backend->getModuleSubjects($module)));
        $var7->setDefault(array_keys($backend->getModuleUsers($module)));

        $this->addHidden($var1);
        $this->addVariable($var2);
        $this->addVariable($var3);
        $this->addVariable($var4);
        $this->addVariable($var5);
        $this->addVariable($var6);
        $this->addVariable($var7);
        $this->addVariable($var8);
        $this->addVariable($var9);
    }
    
}

class DeleteModuleForm extends Form {

    function DeleteModuleForm(&$vars)
    {
        global $backend;
        
        $this->Form($vars, 'deletemoduleform');
        
        $module = $vars->getVar('module');
        $info = $backend->getModule($module);

        $yesnotype = new EnumType(array(0 => _("No"), 1 => _("Yes")));

        $var1 = new Variable('', 'module', getIntType(), true, true);
        $var2 = new Variable(_("Module Name"), 'name', getTextType(), false, true);
        $var3 = new Variable(_("Module Description"), 'description', getTextType(), false, true);
        $var4 = new Variable(_("Really delete this module? This may cause data problems!!"), 'yesno', $yesnotype, true);

        $var2->setDefault($info['name']);
        $var3->setDefault($info['description']);

        $this->addHidden($var1);
        $this->addVariable($var2);
        $this->addVariable($var3);
        $this->addVariable($var4);
    }

}

class AddUserForm extends Form {

    function AddUserForm(&$vars)
    {
        $this->Form($vars, 'adduserform');

        $var1 = new Variable('', 'module', getIntType(), true, true);
        $var2 = new Variable(_("User"), 'user', getTextType(), true);

        $this->addHidden($var1);
        $this->addVariable($var2);
    }
    
}

class EditUserStep1Form extends Form {

    function EditUserStep1Form(&$vars)
    {
        global $backend;

        $this->Form($vars, 'edituserstep1form');

        $module = $vars->getVar('module');
        $users = $backend->getModuleUsers($module);
        $usertype = new EnumType($users);

        $var1 = new Variable('', 'module', getIntType(), true, true);
        $var2 = new Variable(_("Users responsible for this module"), 'user', $usertype, true);

        $this->addHidden($var1);
        $this->addVariable($var2);
    }
    
}

class AddTypeStep1Form extends Form {

    function AddTypeStep1Form(&$vars)
    {
        $this->Form($vars, 'addtypestep1form');

        $var1 = new Variable(_("Type Name"), 'name', getTextType(), true);
        $var2 = new Variable(_("Type Description"), 'description', getTextType(), true);

        $this->addVariable($var1);
        $this->addVariable($var2);
    }
    
}

class EditTypeStep1Form extends Form {

    function EditTypeStep1Form(&$vars)
    {
        global $backend;

        $this->Form($vars, 'edittypestep1form');

        $types = $backend->getAllTypes();
        $ttype = new EnumType($types);

        $var1 = new Variable(_("Type Name"), 'type', $ttype, true);

        $this->addVariable($var1);
    }
    
}

class EditTypeStep2Form extends Form {

    function EditTypeStep2Form(&$vars)
    {
        global $backend;

        $this->Form($vars, 'edittypestep2form');

        $type = $vars->getVar('type');
        $info = $backend->getType($type);
        
        $states = $backend->getStates($type);
        $statetype = new SetType($states);
        $statelink = new LinkType(array('text' => _("Edit States"), 'url' => Horde::applicationUrl('admin.php?formname=editstatestep1form&type=' . $type)));
        
        $priorities = $backend->getPriorities($type);
        $prioritytype = new SetType($priorities);
        $prioritylink = new LinkType(array('text' => _("Edit Priorities"), 'url' => Horde::applicationUrl('admin.php?formname=editprioritystep1form&type=' . $type)));

        $var1 = new Variable('', 'type', getIntType(), true, true);
        $var2 = new Variable(_("Type Name"), 'name', getTextType(), true);
        $var3 = new Variable(_("Type Description"), 'description', getTextType(), true);
        $var4 = new Variable(_("States for this Type"), 'state', $statetype, false, true);
        $var5 = new Variable('', 'link', $statelink, false, true);
        $var6 = new Variable(_("Priorities for this Type"), 'priority', $prioritytype, false, true);
        $var7 = new Variable('', 'link', $prioritylink, false, true);
        
        $var2->setDefault($info['name']);
        $var3->setDefault($info['description']);
        $var4->setDefault(array_keys($states));
        $var6->setDefault(array_keys($priorities));

        $this->addHidden($var1);
        $this->addVariable($var2);
        $this->addVariable($var3);
        $this->addVariable($var4);
        $this->addVariable($var5);
        $this->addVariable($var6);
        $this->addVariable($var7);
    }
    
}

class DeleteTypeForm extends Form {

    function DeleteTypeForm(&$vars)
    {
        global $backend;
        
        $this->Form($vars, 'deletetypeform');
        
        $type = $vars->getVar('type');
        $info = $backend->getType($type);

        $yesnotype = new EnumType(array(0 => _("No"), 1 => _("Yes")));

        $states = $backend->getStates($type);
        $statetype = new SetType($states);
        $statelink = new LinkType(array('text' => _("Edit States"), 'url' => Horde::applicationUrl('admin.php?formname=editstatestep1form&type=' . $type)));

        $priorities = $backend->getPriorities($type);
        $prioritytype = new SetType($priorities);
        $prioritylink = new LinkType(array('text' => _("Edit Priorities"), 'url' => Horde::applicationUrl('admin.php?formname=editprioritystep1form&type=' . $type)));

        $var1 = new Variable('', 'type', getIntType(), true, true);
        $var2 = new Variable(_("Type Name"), 'name', getTextType(), false, true);
        $var3 = new Variable(_("Type Description"), 'description', getTextType(), false, true);
        $var4 = new Variable(_("States for this Type"), 'state', $statetype, false, true);
        $var5 = new Variable(_("Priorities for this Type"), 'priority', $prioritytype, false, true);
        $var6 = new Variable(_("Really delete this type? This may cause data problems!!"), 'yesno', $yesnotype, true);

        $var2->setDefault($info['name']);
        $var3->setDefault($info['description']);
        $var4->setDefault(array_keys($states));
        $var5->setDefault(array_keys($priorities));

        $this->addHidden($var1);
        $this->addVariable($var2);
        $this->addVariable($var3);
        $this->addVariable($var4);
        $this->addVariable($var5);
        $this->addVariable($var6);
    }

}

class AddStateForm extends Form {

    function AddStateForm(&$vars)
    {
        global $backend;

        $this->Form($vars, 'addstateform');

        $categories = $backend->getCategories();
        $ctype = new EnumType($categories);

        $var1 = new Variable('', 'type', getIntType(), true, true);
        $var2 = new Variable(_("State Name"), 'name', getTextType(), true);
        $var3 = new Variable(_("State Description"), 'description', getTextType(), true);
        $var4 = new Variable(_("State Category"), 'category', $ctype, true);

        $this->addHidden($var1);
        $this->addVariable($var2);
        $this->addVariable($var3);
        $this->addVariable($var4);
    }
    
}

class EditStateStep1Form extends Form {

    function EditStateStep1Form(&$vars)
    {
        global $backend;

        $this->Form($vars, 'editstatestep1form');

        $states = $backend->getStates($vars->getVar('type'));
        $stype = new EnumType($states);

        $var1 = new Variable('', 'type', getIntType(), true, true);
        $var2 = new Variable(_("State Name"), 'state', $stype, true);

        $this->addHidden($var1);
        $this->addVariable($var2);
    }
    
}

class EditStateStep2Form extends Form {

    function EditStateStep2Form(&$vars)
    {
        global $backend;

        $this->Form($vars, 'editstatestep2form');

        $state = $vars->getVar('state');
        $info = $backend->getState($state);

        $categories = $backend->getCategories();
        $ctype = new EnumType($categories);
        
        $var1 = new Variable('', 'type', getIntType(), true, true);
        $var2 = new Variable('', 'state', getIntType(), true, true);
        $var3 = new Variable(_("State Name"), 'name', getTextType(), true);
        $var4 = new Variable(_("State Description"), 'description', getTextType(), true);
        $var5 = new Variable(_("State Category"), 'category', $ctype, true);

        $var3->setDefault($info['name']);
        $var4->setDefault($info['description']);
        $var5->setDefault($info['category']);

        $this->addHidden($var1);
        $this->addHidden($var2);
        $this->addVariable($var3);
        $this->addVariable($var4);
        $this->addVariable($var5);
    }
    
}

class DeleteStateForm extends Form {

    function DeleteStateForm(&$vars)
    {
        global $backend;
        
        $this->Form($vars, 'deletestateform');
        
        $state = $vars->getVar('state');
        $info = $backend->getState($state);

        $states = array(0 => _("No"), 1 => _("Yes"));
        $yesnotype = new EnumType($states);

        $var1 = new Variable('', 'type', getIntType(), true, true);
        $var2 = new Variable('', 'state', getIntType(), true, true);
        $var3 = new Variable(_("State Name"), 'name', getTextType(), false, true);
        $var4 = new Variable(_("State Description"), 'description', getTextType(), false, true);
        $var5 = new Variable(_("Really delete this state? This may cause data problems!!"), 'yesno', $yesnotype, true);

        $var3->setDefault($info['name']);
        $var4->setDefault($info['description']);

        $this->addHidden($var1);
        $this->addHidden($var2);
        $this->addVariable($var3);
        $this->addVariable($var4);
        $this->addVariable($var5);
    }

}

class AddPriorityForm extends Form {

    function AddPriorityForm(&$vars)
    {
        $this->Form($vars, 'addpriorityform');

        $var1 = new Variable('', 'type', getIntType(), true, true);
        $var2 = new Variable(_("Priority Name"), 'name', getTextType(), true);
        $var3 = new Variable(_("Priority Description"), 'description', getTextType(), true);

        $this->addHidden($var1);
        $this->addVariable($var2);
        $this->addVariable($var3);
    }
    
}

class EditPriorityStep1Form extends Form {

    function EditPriorityStep1Form(&$vars)
    {
        global $backend;

        $this->Form($vars, 'editprioritystep1form');

        $priorities = $backend->getPriorities($vars->getVar('type'));
        $stype = new EnumType($priorities);

        $var1 = new Variable('', 'type', getIntType(), true, true);
        $var2 = new Variable(_("Priority Name"), 'priority', $stype, true);

        $this->addHidden($var1);
        $this->addVariable($var2);
    }
    
}

class EditPriorityStep2Form extends Form {

    function EditPriorityStep2Form(&$vars)
    {
        global $backend;

        $this->Form($vars, 'editprioritystep2form');

        $priority = $vars->getVar('priority');
        $info = $backend->getPriority($priority);

        $var1 = new Variable('', 'type', getIntType(), true, true);
        $var2 = new Variable('', 'priority', getIntType(), true, true);
        $var3 = new Variable(_("Priority Name"), 'name', getTextType(), true);
        $var4 = new Variable(_("Priority Description"), 'description', getTextType(), true);

        $var3->setDefault($info['name']);
        $var4->setDefault($info['description']);

        $this->addHidden($var1);
        $this->addHidden($var2);
        $this->addVariable($var3);
        $this->addVariable($var4);
    }
    
}

class DeletePriorityForm extends Form {

    function DeletePriorityForm(&$vars)
    {
        global $backend;
        
        $this->Form($vars, 'deletepriorityform');
        
        $priority = $vars->getVar('priority');
        $info = $backend->getPriority($priority);

        $yesnotype = new EnumType(array(0 => _("No"), 1 => _("Yes")));

        $var1 = new Variable('', 'type', getIntType(), true, true);
        $var2 = new Variable('', 'priority', getIntType(), true, true);
        $var3 = new Variable(_("Priority Name"), 'name', getTextType(), false, true);
        $var4 = new Variable(_("Priority Description"), 'description', getTextType(), false, true);
        $var5 = new Variable(_("Really delete this priority? This may cause data problems!!"), 'yesno', $yesnotype, true);

        $var3->setDefault($info['name']);
        $var4->setDefault($info['description']);

        $this->addHidden($var1);
        $this->addHidden($var2);
        $this->addVariable($var3);
        $this->addVariable($var4);
        $this->addVariable($var5);
    }

}
?>
