<?php
/*
 * $Horde: whups/lib/Driver.php,v 1.16 2002/01/29 20:05:24 chuck Exp $
 *
 * Copyright 2001 Robert E. Coyle <robertecoyle@hotmail.com>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/bsdl.php.
 */

// ABC for backends

// TODO: Needs updating to include method stubs for all required methods
// (just to indicate what methods need implementing by other backends)

/**
 * Whups_Driver Class
 *
 * @author Robert E. Coyle <robertecoyle@hotmail.com>
 * @version $Revision: 1.16 $
 * @package whups
 */
class Whups_Driver {
    
    function Whups_Driver()
    {
    }

    function initialise() {}
    function addTicket($requester, $owner, $type, $state, $priority, $module) {}
    function &getTicketsByProperties(&$vars) {}

    /**
     * Attempts to return a concrete Driver instance based on $driver.
     * 
     * @param string $driver The type of concrete Driver subclass to return.
     *                       This is based on the storage driver ($driver). The
     *                       code is dynamically included.
     * @param optional array $params A hash containing any additional
     *                               configuration or connection parameters a
     *                               subclass might need.
     *
     * @return object Driver The newly created concrete Driver instance, or false
     *         on an error.
     */
    function &factory($driver, $params = array())
    {
        $driver = strtolower(basename($driver));
        include_once dirname(__FILE__) . '/Driver/' . $driver . '.php';
        $class = 'Whups_Driver_' . $driver;
        if (class_exists($class)) {
            return new $class($params);
        } else {
            Horde::fatal(sprintf(_("No such backend '%s' found"), $conf['backend']['type']), __FILE__, __LINE__);
        }
    }

    function getCategories()
    {
        return array('unconfirmed' => _("Unconfirmed"),
                     'new' => _("New"),
                     'assigned' => _("Assigned"),
                     'resolved' => _("Resolved"));
    }

    function notifyComment($ticket_id, $comment, $creator) {
        $cname = Auth::getAuth();
        $cemail = '';

        $subject = sprintf(_("New comment for ticket %s"), $ticket_id);
        $message1 = sprintf(_("%s <%s> added the following comment to ticket %s:"), $cname, $cemail, $ticket_id);
        $message1 = $this->_splitMessage($message1);
        $message2 = $this->_splitMessage($comment);

        $message = $message1 . "\n\n" . $message2 . "\n\n";

        $this->_disclaimer($message);
        $this->_sendToAll($ticket_id, $subject, $message);
    }

    function notifyAssignment(&$olddetails, &$comment, $ownername, $owneremail, $newstate) {
        $myname = Auth::getAuth();
        $myemail = '';

        $ticket_id = $olddetails['id'];
        $state_name = $this->getStateName($olddetails['type'], $newstate);

        $message2 = empty($comment) ? "\n\n" : ("\n\n" . $this->_splitMessage($comment) . "\n\n");

        $subject = sprintf(_("Ticket %s has been assigned"), $ticket_id);
        $message = sprintf(_("%s <%s> has assigned ticket %s to %s <%s>. "), $myname, $myemail, $ticket_id, $ownername, $owneremail);
        $message .= sprintf(_("It is now in the %s state."), $state_name);
        $message = $this->_splitMessage($message);
        $message .= $message2;

        $this->_disclaimer($message);
        $this->_sendToAllButOwner($ticket_id, $subject, $message);

        $subject = sprintf(_("You have been assigned ticket %s"), $ticket_id);
        $message = sprintf(_("%s <%s> has assigned ticket %s to you.\n"), $myname, $myemail, $ticket_id);
        $message .= sprintf(_("It is now in the %s state."), $state_name);
        $message .= $message2;

        $this->_disclaimer($message);
        $this->_mail("$ownername <$owneremail>", $subject, $message);
    }

    function notifyStateChange($ticket_id, $type, $tostate, $comment, $userid) {
        $statename = $this->getStateName($type, $tostate);

        $myname = Auth::getAuth();
        $myemail = '';

        $subject = sprintf(_("Ticket %s has changed state"), $ticket_id);
        $message = sprintf(_("%s <%s> has changed the state of ticket %s to state %s. "), $myname, $myemail, $ticket_id, $statename);
        $message .= _("The explanation was:");
        $message = $this->_splitMessage($message);
        $message .= "\n\n";

        $message .= $this->_splitMessage($comment);
        $message .= "\n\n";

        $this->_disclaimer($message);
        $this->_sendToAll($ticket_id, $subject, $message);
    }

    function notifyPriorityChange($ticket_id, $type, $priority, $comment, $userid) {
        $priorityname = $this->getPriorityName($type, $priority);

        $myname = Auth::getAuth();
        $myemail = '';

        $subject = sprintf(_("Ticket %s has changed priority"), $ticket_id);
        $message = sprintf(_("%s <%s> has changed the priority of ticket %s to %s. "), $myname, $myemail, $ticket_id, $priorityname);
        $message .= _("The explanation was:");
        $message = $this->_splitMessage($message);
        $message .= "\n\n";

        $message .= $this->_splitMessage($comment);
        $message .= "\n\n";

        $this->_disclaimer($message);
        $this->_sendToAll($ticket_id, $subject, $message);
    }

    function _disclaimer(&$message) {
        $message1 = _("This mail was automatically generated by the Whups request tracking system");
        $message1 .= " (http://horde.org/whups/).";

        $message2 = sprintf(_("If you have recieved this mail in error, please contact %s."), $GLOBALS['conf']['mail']['contact_addr']);

        $message .= "-- \n";
        $message .= $this->_splitMessage($message1);
        $message .= "\n\n";
        $message .= $this->_splitMessage($message2);
        $message .= "\n";
    }

    function _sendToAllButOwner($ticket_id, $subject, $message) {
        $addresses = $this->getListeners($ticket_id, false);
        
        /*
        foreach ($addresses as $key => $info) {
            $name = $info['name'];
            $email = $info['email'];

            $this->_mail("$name <$email>", $subject, $message);
        }
        */
    }

    function _sendToAll($ticket_id, $subject, $message) {
        $addresses = $this->getListeners($ticket_id);

        /*
        foreach ($addresses as $key => $info) {
            $name = $info['name'];
            $email = $info['email'];

            $this->_mail("$name <$email>", $subject, $message);
        }
        */
    }

    function _mail($to, $subject, $message) {
        $subject = "[Whups] $subject";

        echo "<font color=\"white\"><pre>\n";
        echo sprintf(_("Sending mail to %s\n"), htmlspecialchars($to));
        echo _("Subject: ") . htmlspecialchars($subject) . "\n\n";
        echo htmlspecialchars($message) . "\n";
        echo "</pre></font>\n";
    }

    function _splitMessage($message) {
        $set    = split("[ \t]+", $message);
        $len    = 0;
        $result = '';

        foreach ($set as $substr) {
            while (($pos = strpos($substr, "\n")) !== false) {
                $result .= substr($substr, 0, $pos);
                $len = 0;
                $substr = substr($substr, $pos + 1);
            }

            $slen = strlen($substr);

            if (!$slen) {
                continue;
            }

            $len += $slen + 1;

            if ($len > 80) {
                $len = $slen;
                $result .= "\n" . $substr . ' ';
            } else {
                $result .= $substr . ' ';
            }
        }

        return $result;
    }

}
?>
