<?php
/*
 * $Horde: whups/lib/Driver/sql.php,v 1.29 2002/03/02 19:16:40 chuck Exp $
 *
 * Copyright 2001 Robert E. Coyle <robertecoyle@hotmail.com>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/bsdl.php.
 */

/**
 * Whups_Driver_sql Class
 *
 * @author Robert E. Coyle <robertecoyle@hotmail.com>
 * @version $Revision: 1.29 $
 * @package whups
 */
class Whups_Driver_sql extends Whups_Driver
{
    function initialise()
    {
        global $conf;

        @include_once 'DB.php';
        $this->db = &DB::connect($conf['driver']['dsn'], true);

        if (DB::isError($this->db) || DB::isWarning($this->db)) {
            Horde::fatal($this->db, __FILE__, __LINE__);
        }

        /* Enable the "portability" option. */
        $this->db->setOption('optimize', 'portability');

        return true;
    }

    function addModule($name, $description)
    {
        // Get a new unique id.
        $new_id = $this->db->nextId('whups_modules');
        if (DB::isError($new_id)) {
            return $new_id;
        }

        $result = $this->_query('INSERT INTO whups_modules (module_id, module_name, module_description) VALUES (' .
                                $this->db->quote($new_id) . ', ' . $this->db->quote($name) . ', ' . $this->db->quote($description) . ')');
        if (DB::isError($result)) {
            return $result;
        }

        return $new_id;
    }

    function addSubject($name)
    {
        // Get a new unique id.
        $new_id = $this->db->nextId('whups_subjects');
        if (DB::isError($new_id)) {
            return $new_id;
        }

        $result = $this->_query('INSERT INTO whups_subjects (subject_id, subject_name) VALUES (' .
                                $this->db->quote($new_id) . ', ' . $this->db->quote($name) . ')');
        if (DB::isError($result)) {
            return $result;
        }

        return $new_id;
    }

    function addType($name, $description)
    {
        // Get a new unique id.
        $new_id = $this->db->nextId('whups_types');
        if (DB::isError($new_id)) {
            return $new_id;
        }

        $result = $this->_query('INSERT INTO whups_types (type_id, type_name, type_description) VALUES (' .
                                $this->db->quote($new_id) . ', ' . $this->db->quote($name) . ', ' . $this->db->quote($description) . ')');
        if (DB::isError($result)) {
            return $result;
        }

        return $new_id;
    }

    function addState($typeID, $name, $description, $category)
    {
        // Get a new state id.
        $new_id = $this->db->nextId('whups_states');
        if (DB::isError($new_id)) {
            return $new_id;
        }

        $result = $this->_query('INSERT INTO whups_states (state_id, type_id, state_name, state_description, state_category) VALUES (' .
                                $this->db->quote($new_id) . ', ' . $this->db->quote($typeID) . ', ' . $this->db->quote($name) . ', ' . $this->db->quote($description) . ', ' . $this->db->quote($category) . ')');
        if (DB::isError($result)) {
            return $result;
        }

        return $new_id;
    }

    function addPriority($typeID, $name, $description)
    {
        // Get a new priority id.
        $new_id = $this->db->nextId('whups_priorities');
        if (DB::isError($new_id)) {
            return $new_id;
        }

        $result = $this->_query('INSERT INTO whups_priorities (priority_id, type_id, priority_name, priority_description) VALUES (' .
                                $this->db->quote($new_id) . ', ' . $this->db->quote($typeID) . ', ' . $this->db->quote($name) . ', ' . $this->db->quote($description) . ')');
        if (DB::isError($result)) {
            return $result;
        }

        return $new_id;
    }

    function addTicket(&$info)
    {
        // get our vars
        $requester = Auth::getAuth();
        $type = $info['type'];
        $state = $info['state'];
        $priority = $info['priority'];
        $module = $info['module'];
        $summary = $info['summary'];
        $comment = $info['comment'];

        // Get the new unique ids for this ticket and the initial comment.
        $new_id = $this->db->nextId('whups_tickets');

        if (DB::isError($new_id)) {
            return false;
        }

        $new_comment_id = $this->db->nextId('whups_comments');

        if (DB::isError($new_comment_id)) {
            return false;
        }

        // create the ticket
        $result = $this->_query("INSERT INTO whups_tickets (ticket_id, ticket_summary, user_id_requester, user_id_owner, type_id, state_id, " .
                                "    priority_id, module_id, comment_id, ticket_timestamp) " .
                                "VALUES ($new_id, " . $this->db->quote($summary) . ', ' . $this->db->quote($requester) . ", 0, $type, $state," .
                                "    $priority, $module, $new_comment_id, " . time() . ')');

        if (DB::isError($result)) {
            Horde::fatal($result, __FILE__, __LINE__);
        }

        // create the initial comment
        $result = $this->_query("INSERT INTO whups_comments (comment_id, ticket_id, user_id_creator, comment_text, comment_timestamp) " .
                                "VALUES ($new_comment_id, $new_id, " . $this->db->quote($requester) . ', ' . $this->db->quote($comment) . ', ' . time() . ')');

        if (DB::isError($result)) {
            Horde::fatal($result, __FILE__, __LINE__);
        }

        $result = $this->_updateStateLog($new_id, 'NULL', $state, $requester, $new_comment_id);

        if (DB::isError($result)) {
            Horde::fatal($result, __FILE__, __LINE__);
        }

        return array($new_id, $new_comment_id);
    }

    function addComment(&$info)
    {
        $ticket_id = $info['id'];
        $comment = $info['newcomment'];
        $creator = Auth::getAuth();

        $result = $this->_addComment($ticket_id, $comment, $creator);

        $this->notifyComment($ticket_id, $comment, $creator);
    }

    function assignTicket(&$info)
    {
        $ticket_id = $info['id'];
        $assignee = $this->db->quote($info['assignee']);
        $state = $info['tostate'];
        $morecomment = $info['newcomment'];

        $user_name = $assignee;
        $user_email = '';

        $comment = sprintf(_("Assigned ticket to %s <%s>"), $user_name, $user_email);

        if (!empty($morecomment)) {
            $comment .= "\n\n$morecomment";
        }

        $ticket_details = $this->getTicketDetails($ticket_id);

        $this->_query("UPDATE whups_tickets SET user_id_owner = $assignee, state_id = $state WHERE ticket_id = $ticket_id");
        $new_comment_id = $this->_addComment($ticket_id, $comment, Auth::getAuth());
        $this->_updateStateLog($ticket_id, $ticket_details['state'], $state, Auth::getAuth(), $new_comment_id);

        $this->notifyAssignment($ticket_details, $morecomment, $user_name, $user_email, $state);
    }

    function setTicketState(&$info)
    {
        $ticket_id = $info['id'];
        $newcomment = $info['newcomment'];
        $tostate = $info['state'];
        $type = $info['type'];

        $this->_query("UPDATE whups_tickets SET state_id = $tostate WHERE ticket_id = $ticket_id");
        $new_comment_id = $this->_addComment($ticket_id, $newcomment, Auth::getAuth());
        $this->_updateStateLog($ticket_id, '', $tostate, Auth::getAuth(), $new_comment_id);

        $this->notifyStateChange($ticket_id, $type, $tostate, $newcomment, Auth::getAuth());
    }

    function setTicketPriority(&$info)
    {
        $ticket_id = $info['id'];
        $newcomment = $info['newcomment'];
        $priority = $info['priority'];
        $type = $info['type'];

        $this->_query("UPDATE whups_tickets SET priority_id = $priority WHERE ticket_id = $ticket_id");
        $new_comment_id = $this->_addComment($ticket_id, $newcomment, Auth::getAuth());

        $this->notifyPriorityChange($ticket_id, $type, $priority, $newcomment, Auth::getAuth());
    }

    function &executeQuery(&$query)
    {
        // This isn't reentrant (should pass $this->tables to query->reduce and have that pass it to the callback), 
        // but I don't think that this is a problem in php
        $this->tables = array();

        $where = 'where ' . $query->reduce($this, '_clauseFromQuery');
        $from = 'from ' . implode(', ', array_keys($this->tables));

        $sql = "select ticket.id $from $where";

        // TODO: Actually execute the query...
        debug("Type this into your nearest sql prompt :-)<br />");
        debug($sql);
    }

    function _clauseFromQuery(&$args, $type, $ttype, $criterion, $operator, $value)
    {
        switch ($type) {
        case QUERY_TYPE_AND:
            $result = $this->_concatClauses($args, 'and');
            break;

        case QUERY_TYPE_OR:
            $result = $this->_concatClauses($args, 'or');
            break;

        case QUERY_TYPE_NOT:
            $result = $this->_notClause($args);
            break;

        case QUERY_TYPE_CRITERION:
            $result = $this->_criterionClause($ttype, $criterion, $operator, $value);
            break;
        }

        return $result;
    }

    function _concatClauses(&$args, $conjunction)
    {
        $count = count($args);

        if ($count == 0) {
            $result = '';
        } else if ($count == 1) {
            $result = $args[0];
        } else {
            $result = '(' . $args[0] . ')';

            for ($i = 1; $i < $count; $i++) {
                if ($args[$i] != '') {
                    $result .= ' ' . $conjunction . ' (' . $args[$i] . ')';
                }
            }
        }

        return $result;
    }

    function _notClause($args)
    {
        if (count($args) == 0) {
            return '';
        }

        // TODO: put in a sanity check: count($args) should be 1 always
        return 'not (' . $args[0] . ')';
    }

    function _criterionClause($ttype, $criterion, $operator, $value)
    {
        $includeType = false;

        $this->tables['ticket'] = 1;

        switch ($operator) {
        case OPERATOR_GREATER:         $op = '>'; break;
        case OPERATOR_LESS:            $op = '<'; break;
        case OPERATOR_EQUAL:           $op = '='; break;
        case OPERATOR_CI_SUBSTRING:    $op = _(" contains (case insensitive) substring"); break;
        case OPERATOR_CS_SUBSTRING:    $op = _(" contains (case sensitive) substring"); break;
        case OPERATOR_WORD:            $op = _(" contains the word"); break;
        case OPERATOR_PATTERN:         $op = _("like"); break;
        }

        if (!preg_match("/^[0-9]+$/", $value)) {
            $value = $this->db->quote($value);
        }

        $done = false;

        switch ($criterion) {
        case CRITERION_ID:             $text = "ticket.id"; break;
        case CRITERION_MODULE:         $text = "ticket.module"; break;
        case CRITERION_STATE:          $text = "ticket.state"; $includeType = true; break;
        case CRITERION_PRIORITY:       $text = "ticket.priority"; $includeType = true; break;
        case CRITERION_CATEGORY:       $text = "Category"; break;
        case CRITERION_OWNER:          $text = "ticket.owner"; break;
        case CRITERION_REQUESTER:      $text = "ticket.requester"; break;
        case CRITERION_QA:             $text = "QA contact"; break;
        case CRITERION_CC:             $text = "CC list"; break;
        case CRITERION_ADDED_COMMENT:  $text = "Commentor"; break;
        case CRITERION_SUMMARY:        $text = "ticket.ticket_summary"; break;

        case CRITERION_COMMENT:
            $this->tables['comment'] = 1;
            $text = "(ticket.id = comment.id and comment.comment $op $value)";
            $done = true;
            break;
        }

        if ($done == false) {
            $text = "$text $op $value";
        }

        if ($includeType == true) {
            $text = "(ticket.type = $ttype and $text)";
        }

        return $text;
    }

    function &getTicketsByProperties(&$info, $munge = true)
    {
        // search conditions
        $where = $this->_generateWhere(
            'whups_tickets',
            array('ticket_id', 'type_id', 'state_id', 'priority_id', 'module_id'),
            $info, 'integer');

        $where2 = $this->_generateWhere('whups_tickets', array('user_id_owner', 'user_id_requester'), $info, 'string');

        if (empty($where)) {
            $where = $where2;
        } else if (!empty($where2)) {
            $where .= ' AND ' . $where2;
        }

        $fields = array('ticket_id as id',
			'ticket_summary as summary',
			'user_id_requester',
			'user_id_owner',
			'state_id as state',
			'type_id as type',
			'priority_id as priority',
			'module_id as module',
			'comment_id');

        $fields = $this->_prefixTableToColumns('whups_tickets', $fields);
        $fields .= ', whups_tickets.ticket_timestamp AS timestamp';
        $tables = 'whups_tickets';

        $nouc = isset($info['nouc']) ? $info['nouc'] : null;
        $nores = isset($info['nores']) ? $info['nores'] : null;
        $nonew = isset($info['nonew']) ? $info['nonew'] : null;
        $noass = isset($info['noass']) ? $info['noass'] : null;

        if ($nouc || $nores || $nonew || $noass) {
            $nouc = $nouc ? " AND whups_states.state_category <> 'unconfirmed'" : '';
            $nores = $nores ? " AND whups_states.state_category <> 'resolved'" : '';
            $nonew = $nonew ? " AND whups_states.state_category <> 'new'" : '';
            $noass = $noass ? " AND whups_states.state_category <> 'assigned'" : '';

            $where = $this->_addWhere($where, 1, "(whups_tickets.type_id = whups_states.type_id AND whups_tickets.state_id = whups_states.state_id$nouc$nores$nonew$noass)");
            $tables .= ', whups_states';
            $havestate = 1;
        }

        if ($munge) {
            $fields = "$fields, " .
                "whups_types.type_name AS _type_name, " .
                "whups_states.state_name AS _state_name, " .
                "whups_modules.module_name AS _module_name, " .
                "whups_priorities.priority_name AS _priority_name";

            $tables = "$tables, whups_types, whups_modules, whups_priorities" . (!empty($havestate) ? '' : ', whups_states');

            $where = $this->_addWhere($where, 1,
                "(whups_tickets.type_id = whups_types.type_id) AND " .
                (!empty($havestate) ? '' : "(whups_tickets.type_id = whups_states.type_id AND whups_tickets.state_id = whups_states.state_id) AND ") .
                "(whups_tickets.module_id = whups_modules.module_id) AND " .
                "(whups_tickets.type_id = whups_priorities.type_id AND whups_tickets.priority_id = whups_priorities.priority_id)");
        }

        return $this->_getAll(
            "SELECT $fields FROM $tables WHERE $where ORDER BY whups_tickets.priority_id, whups_tickets.ticket_timestamp");
    }

    function getTicketDetails($ticket)
    {
        $info = array('id' => $ticket);

        $result = $this->getTicketsByProperties($info);
        $result = $result[0];

        if (!empty($result['user_id_requester'])) {
            $result['_requester_name'] = $result['user_id_requester'];
            $result['_requester'] = $result['user_id_requester'];
        }

        if (!empty($result['user_id_owner'])) {
            $result['_owner_name'] = $result['user_id_owner'];
            $result['_owner_email'] = '';
            $result['_owner'] = $result['user_id_owner'];
        }

        return $result;
    }

    function getComments($ticket, $munge = true)
    {
        // search conditions
        $where = "ticket_id = $ticket";
        $tables = 'whups_comments';

        $fields = array('comment_id', 'ticket_id', 'user_id_creator', 'comment_text');
        $fields = $this->_prefixTableToColumns('whups_comments', $fields);
        $fields .= ', comment_timestamp AS timestamp';

        $results = $this->_getAll("SELECT $fields FROM $tables WHERE $where ORDER BY whups_comments.comment_timestamp");

        if (!$munge) {
            return $results;
        }

        // TODO: get attachments
        return $results;
    }

    function getModule($moduleID)
    {
        $module = $this->_getAssoc('SELECT module_id, module_name, module_description, module_subjectlist FROM whups_modules WHERE module_id = ' . $this->db->quote($moduleID));
        if (DB::isError($module) || count($module) == 0) {
            return $module;
        }
        return array('id' => $moduleID,
                     'name' => $module[$moduleID][0],
                     'description' => $module[$moduleID][1],
                     'subjectlist' => $module[$moduleID][2]);
    }

    function getModules()
    {
        return $this->_getAssoc('SELECT module_id, module_name FROM whups_modules');
    }

    function updateModule($moduleID, $name, $description, $types, $subjectlist, $subjects)
    {
        // first update the module entry itself
        $result = $this->_query('UPDATE whups_modules SET' .
                                ' module_name = ' . $this->db->quote($name) . ',' .
                                ' module_description = ' . $this->db->quote($description) . ',' .
                                ' module_subjectlist = ' . (is_null($subjectlist) ? 0 : 1) .
                                ' WHERE module_id = ' . $this->db->quote($moduleID));
        if (PEAR::isError($result)) {
            return $result;
        }

        // clear all previous type-module associations
        $result = $this->_query('DELETE FROM whups_types_modules WHERE module_id = ' . $this->db->quote($moduleID));
        if (PEAR::isError($result)) {
            return $result;
        }

        // add the new associations
        foreach ($types as $typeID) {
            $result = $this->_query('INSERT INTO whups_types_modules (module_id, type_id) VALUES (' .
                                    $this->db->quote($moduleID) . ', ' . $this->db->quote($typeID) . ')');
            if (PEAR::isError($result)) {
                return $result;
            }
        }

        // clear all previous subject-module associations
        $result = $this->_query('DELETE FROM whups_modules_subjects WHERE module_id = ' . $this->db->quote($moduleID));
        if (PEAR::isError($result)) {
            return $result;
        }

        // add the new associations
        foreach ($subjects as $subjectID) {
            $result = $this->_query('INSERT INTO whups_modules_subjects (module_id, subject_id) VALUES (' .
                                    $this->db->quote($moduleID) . ', ' . $this->db->quote($subjectID) . ')');
            if (PEAR::isError($result)) {
                return $result;
            }
        }

        // whew, everything worked
        return true;
    }

    function deleteModule($moduleID)
    {
        return $this->_query('DELETE FROM whups_modules WHERE module_id = ' . $this->db->quote($moduleID));
    }

    function getSubject($subjectID)
    {
        $subject = $this->_getAssoc('SELECT subject_id, subject_name FROM whups_subjects WHERE subject_id = ' . $this->db->quote($subjectID));
        if (DB::isError($subject)) {
            return $subject;
        }
        return array('id' => $subjectID,
                     'name' => $subject[$subjectID]);
    }

    function getSubjects()
    {
        return $this->_getAssoc('SELECT subject_id, subject_name FROM whups_subjects ORDER BY subject_name');
    }

    function getModuleSubjects($moduleID)
    {
        return $this->_getAssoc('SELECT s.subject_id, s.subject_name FROM whups_modules_subjects sm, whups_subjects s' .
                                ' WHERE sm.module_id = ' . $this->db->quote($moduleID) . ' AND sm.subject_id = s.subject_id');
    }

    function getModuleSubjectNames($moduleID)
    {
        return $this->_getAssoc('SELECT s.subject_name, s.subject_name FROM whups_modules_subjects sm, whups_subjects s' .
                                ' WHERE sm.module_id = ' . $this->db->quote($moduleID) . ' AND sm.subject_id = s.subject_id');
    }

    function updateSubject($subjectID, $name)
    {
        // first update the subject entry itself
        $result = $this->_query('UPDATE whups_subjects SET' .
                                ' subject_name = ' . $this->db->quote($name) .
                                ' WHERE subject_id = ' . $this->db->quote($subjectID));

        if (PEAR::isError($result)) {
            return $result;
        }

        return true;
    }

    function deleteSubject($subjectID)
    {
        return $this->_query('DELETE FROM whups_subjects WHERE subject_id = ' . $this->db->quote($subjectID));
    }

    function getModuleUsers($moduleID)
    {
        return $this->_getAssoc('SELECT user_uid, user_uid FROM whups_modules_users' .
                                ' WHERE module_id = ' . $this->db->quote($moduleID));
    }

    function addModuleUser($moduleID, $userID)
    {
        return $this->_query('INSERT INTO whups_modules_users (module_id, user_uid) values (' .
                             $this->db->quote($moduleID) . ', ' . $this->db->quote($userID) . ')');
    }

    function removeModuleUser($moduleID, $userID)
    {
        return $this->_query('DELETE FROM whups_modules_users' .
                             ' WHERE module_id = ' . $this->db->quote($moduleID) .
                             ' AND user_uid = ' . $this->db->quote($userID));
    }

    function updateModuleUsers($moduleID, $users)
    {
        // clear all previous type-module associations
        $result = $this->_query('DELETE FROM whups_types_modules WHERE module_id = ' . $this->db->quote($moduleID));
        if (PEAR::isError($result)) {
            return $result;
        }

        // add the new associations
        foreach ($types as $typeID) {
            $result = $this->_query('INSERT INTO whups_types_modules (module_id, type_id) VALUES (' .
                                    $this->db->quote($moduleID) . ', ' . $this->db->quote($typeID) . ')');
            if (PEAR::isError($result)) {
                return $result;
            }
        }

        // whew, everything worked
        return true;
    }

    function getType($typeID)
    {
        if (empty($typeID)) {
            return false;
        }
        $type = $this->_getAssoc('SELECT type_id, type_name, type_description FROM whups_types WHERE type_id = ' . $this->db->quote($typeID));
        if (DB::isError($type)) {
            return $type;
        }
        return array('id' => $typeID,
                     'name' => isset($type[$typeID][0]) ? $type[$typeID][0] : '',
                     'description' => isset($type[$typeID][1]) ? $type[$typeID][1] : '');
    }

    function getTypes($moduleID)
    {
        return $this->_getAssoc('SELECT t.type_id, t.type_name FROM whups_types t, whups_types_modules tm WHERE tm.module_id = ' . $this->db->quote($moduleID) . ' AND tm.type_id = t.type_id');
    }

    function getAllTypes()
    {
        return $this->_getAssoc('SELECT type_id, type_name FROM whups_types');
    }

    function updateType($typeID, $name, $description)
    {
        return $this->_query('UPDATE whups_types SET' .
                             ' type_name = ' . $this->db->quote($name) . ',' .
                             ' type_description = ' . $this->db->quote($description) .
                             ' WHERE type_id = ' . $this->db->quote($typeID));
    }

    function deleteType($typeID)
    {
        return $this->_query('DELETE FROM whups_types WHERE type_id = ' . $this->db->quote($typeID));
    }

    function getStates($type, $category = '', $notcategory = '')
    {
        if (empty($type)) {
            return array();
        }

        $where = "type_id = $type";
        $where = $this->_addWhere($where, $category, 'state_category = ' . $this->db->quote($category));
        $where = $this->_addWhere($where, $notcategory, 'state_category <> ' . $this->db->quote($notcategory));

        return $this->_getAssoc("SELECT state_id, state_name FROM whups_states WHERE $where");
    }

    function getStateName($type, $state)
    {
        return $this->_getOne("SELECT state_name FROM whups_states WHERE type_id = $type AND state_id = $state");
    }

    function getState($stateID)
    {
        if (empty($stateID)) {
            return false;
        }
        $state = $this->_getAssoc('SELECT state_id, state_name, state_description, state_category FROM whups_states WHERE state_id = ' . $this->db->quote($stateID));
        if (DB::isError($state)) {
            return $state;
        }
        return array('id' => $stateID,
                     'name' => isset($state[$stateID][0]) ? $state[$stateID][0] : '',
                     'description' => isset($state[$stateID][1]) ? $state[$stateID][1] : '',
                     'category' => isset($state[$stateID][2]) ? $state[$stateID][2] : '');
    }

    function updateState($stateID, $name, $description, $category)
    {
        return $this->_query('UPDATE whups_states SET' .
                             ' state_name = ' . $this->db->quote($name) . ',' .
                             ' state_description = ' . $this->db->quote($description) . ',' .
                             ' state_category = ' . $this->db->quote($category) .
                             ' WHERE state_id = ' . $this->db->quote($stateID));
    }

    function deleteState($stateID)
    {
        return $this->_query('DELETE FROM whups_states WHERE state_id = ' . $this->db->quote($stateID));
    }

    function getPriorities($type)
    {
        if (empty($type)) {
            return array();
        }
        return $this->_getAssoc("SELECT priority_id, priority_name FROM whups_priorities WHERE type_id = $type");
    }

    function getPriorityName($type, $priority)
    {
        return $this->_getOne("SELECT priority_name FROM whups_priorities WHERE type_id = $type AND priority_id = $priority");
    }

    function getPriority($priorityID)
    {
        if (empty($priorityID)) {
            return false;
        }
        $priority = $this->_getAssoc('SELECT priority_id, priority_name, priority_description FROM whups_priorities WHERE priority_id = ' . $this->db->quote($priorityID));
        if (DB::isError($priority)) {
            return $priority;
        }
        return array('id' => $priorityID,
                     'name' => isset($priority[$priorityID][0]) ? $priority[$priorityID][0] : '',
                     'description' => isset($priority[$priorityID][1]) ? $priority[$priorityID][1] : '');
    }

    function updatePriority($priorityID, $name, $description)
    {
        return $this->_query('UPDATE whups_priorities SET' .
                             ' priority_name = ' . $this->db->quote($name) . ',' .
                             ' priority_description = ' . $this->db->quote($description) .
                             ' WHERE priority_id = ' . $this->db->quote($priorityID));
    }

    function deletePriority($priorityID)
    {
        return $this->_query('DELETE FROM whups_priorities WHERE priority_id = ' . $this->db->quote($priorityID));
    }

    function getUserSearches($user)
    {
        return $this->_getAssoc("SELECT search_name, search_getvars FROM whups_users_searches WHERE user_id = " . $this->db->quote($user));
    }

    function getUserQueries($user) {
        return $this->_getAll("SELECT query_name FROM whups_users_queries WHERE user_id = " . $this->db->quote($user));
    }

    function saveQuery($name, $var)
    {
        $userid = Auth::getAuth();

        $userid = $this->db->quote($userid);
        $name   = $this->db->quote($name);
        $var    = $this->db->quote($var);

        // can't rely on having replace into
        // FIXME: This should be inside a transaction
        // $this->db->setAutoCommit(false);
        $this->_query("DELETE FROM whups_users_queries WHERE user_id = $userid AND query_name = $name");
        $this->_query("INSERT INTO whups_users_queries (user_id, query_name, query_object) VALUES ($userid, $name, $var)");
        // $this->db->commit();
        // $this->db->setAutoCommit(true);
    }

    function &loadQuery($name)
    {
        $userid = Auth::getAuth();

        $userid = $this->db->quote($userid);
        $name   = $this->db->quote($name);

        $result = $this->_getOne("SELECT query_object FROM whups_users_queries WHERE user_id = $userid AND query_name = $name");
        return $result;
    }

    // tricky - handle case where owner = requester
    function getListeners($ticket, $withowner = true, $withrequester = true)
    {
        $tables = "user, whups_tickets_listeners, ticket";
        $where = "(whups_tickets_listeners.ticket_id = $ticket AND user.id = whups_tickets_listeners.user)";

        if (!$withowner) {
            $where .= " AND (ticket.id = $ticket AND ticket.owner <> user.id AND ticket.owner <> ticket.requester)";
        } else {
            $where .= " OR (ticket.id = $ticket AND ticket.owner = user.id)";
        }

        if (!$withrequester) {
            $where .= " AND (ticket.id = $ticket AND ticket.requester <> user.id)";
        } else {
            $where .= " OR (ticket.id = $ticket AND ticket.requester = user.id AND ticket.requester <> ticket.owner)";
        }

        return $this->_getAll("SELECT user.name, user.email FROM $tables WHERE $where GROUP BY user.id");
    }

    // -- private methods --

    function _generateWhere($table, $fields, &$info, $type)
    {
        $where = '';
        $this->_mapFields($info);

        foreach ($fields as $field) {
            if (isset($info[$field])) {
                $prop = $info[$field];
                $success = @settype($prop, $type);
                $where = $this->_addWhere($where, $prop && $success, "$table.$field = " . $this->db->quote($prop));
            }
        }

        foreach ($fields as $field) {
            if (isset($info["not$field"])) {
                $prop = $info["not$field"];

                if (strpos($prop, ',') === false) {
                    $success = @settype($prop, $type);
                    $where = $this->_addWhere($where, $prop && $success, "$table.$field <> " . $this->db->quote($prop));
                } else {
                    $set = explode(',', $prop);

                    foreach ($set as $prop) {
                        $success = @settype($prop, $type);
                        $where = $this->_addWhere($where, $prop && $success, "$table.$field <> " . $this->db->quote($prop));
                    }
                }
            }
        }

        return $where;
    }

    function _mapFields(&$info)
    {
        foreach ($info as $key => $val) {
            if ($key === 'id') {
                $info['ticket_id'] = $info['id'];
                unset($info['id']);
            } elseif ($key === 'state' ||
                      $key === 'type' ||
                      $key === 'module' ||
                      $key === 'priority') {
                $info[$key . '_id'] = $info[$key];
                unset($info[$key]);
            } elseif ($key === 'owner' ||
                      $key === 'requester') {
                $info['user_id_' . $key] = $info[$key];
                unset($info[$key]);
            }
        }
    }

    function _generateTextSearch($field, $tomatch)
    {
        $matches = split('[ \\n\\t]+', $tomatch);

        foreach ($matches as $match) {
            $where = $this->_addWhere($where, 1, "$field LIKE '% $match %'", 'OR');
        }

        return $where;
    }

    function _addWhere($where, $condition, $clause, $conjunction = 'AND')
    {
        if (!empty($condition)) {
            if (!empty($where)) {
                $where .= " $conjunction ";
            }

            $where .= "($clause)";
        }

        return $where;
    }

    function _prefixTableToColumns($table, &$columns)
    {
        $join = "";

        $clause = '';
        foreach ($columns as $column) {
            $clause .= "$join$table.$column";
            $join = ', ';
        }

        return $clause;
    }

    function _updateStateLog($ticket, $fromstate, $tostate, $owner, $comment_id)
    {
        if (empty($fromstate)) {
            $fromstate = $this->_getOne("SELECT state FROM ticket WHERE id = $ticket");
        }

        return $this->_query("INSERT INTO whups_state_logs (ticket_id, state_from, state_to, state_timestamp, user_id_owner, state_explanation) " .
                             "VALUES ($ticket, $fromstate, $tostate, " . time() . ", '$owner', $comment_id)");
    }

    function _addComment($ticket_id, $comment, $creator)
    {
        // get the new id
        $new_comment_id = $this->db->nextId('whups_comments');

        if (DB::isError($new_comment_id))
            return false;

        // set up our vars
        $comment = $this->db->quote($comment);

        // add the row
        $this->_query("INSERT INTO whups_comments (comment_id, ticket_id, user_id_creator, comment_text, comment_timestamp) " .
                      "VALUES ($new_comment_id, $ticket_id, '$creator', $comment, " . time() . ')');

        return $new_comment_id;
    }

    // I can't believe PEAR doesn't support this already
    function &_query($query)
    {
        $start = getmicrotime();
        $result = $this->db->query($query);
        $end = getmicrotime();

        $this->_debugDB($query, $result, $end - $start);
        return $result;
    }

    function &_getAll($query)
    {
        $start = getmicrotime();
        $result = $this->db->getAll($query, null, DB_FETCHMODE_ASSOC);
        $end = getmicrotime();

        $this->_debugDB($query, $result, $end - $start);
        return $result;
    }

    function &_getAssoc($query)
    {
        $start = getmicrotime();
        $result = $this->db->getAssoc($query);
        $end = getmicrotime();

        $this->_debugDB($query, $result, $end - $start);
        return $result;
    }

    function &_getOne($query)
    {
        $start = getmicrotime();
        $result = $this->db->getOne($query);
        $end = getmicrotime();

        $this->_debugDB($query, $result, $end - $start);
        return $result;
    }

    function _debugDB(&$query, &$result, $time)
    {
        global $debug_db;

        if ($debug_db) {
            $time = sprintf("Completed in %.3f seconds.", $time);

            echo "<div class=\"debug\"><b>$query</b><br /><pre>\n";
            print_r($result);
            echo "\n$time\n";
            echo "</pre></div>\n";
        }
    }

}
?>
