<?php
/*
 * $Horde: whups/lib/Query.php,v 1.5 2002/02/20 18:05:26 chuck Exp $
 *
 * Copyright 2001 Robert E. Coyle <robertecoyle@hotmail.com>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/bsdl.php.
 */

// node types
define('QUERY_TYPE_AND',       1);
define('QUERY_TYPE_OR',        2);
define('QUERY_TYPE_NOT',       3);
define('QUERY_TYPE_CRITERION', 4);

// criterion types
define('CRITERION_ID',             1);
define('CRITERION_MODULE',         2);
define('CRITERION_TYPE',           3);
define('CRITERION_STATE',          4);
define('CRITERION_PRIORITY',       5);
define('CRITERION_CATEGORY',       6);
define('CRITERION_OWNER',          7);
define('CRITERION_REQUESTER',      8);
define('CRITERION_QA',             9);
define('CRITERION_CC',            10);
define('CRITERION_ADDED_COMMENT', 11);
define('CRITERION_COMMENT',       12);
define('CRITERION_SUMMARY',       13);

// operators
// integer fields
define('OPERATOR_GREATER', 1);
define('OPERATOR_LESS',    2);
define('OPERATOR_EQUAL',   3);

// text fields
define('OPERATOR_CI_SUBSTRING',  4);
define('OPERATOR_CS_SUBSTRING',  5);
define('OPERATOR_WORD',          6);
define('OPERATOR_PATTERN',       7);

$textOperators = array(
    OPERATOR_EQUAL        => _("Exact Match"),
    OPERATOR_CI_SUBSTRING => _("Case Insensitive Substring"),
    OPERATOR_CS_SUBSTRING => _("Case Sensitive Substring"),
    OPERATOR_WORD         => _("Match Word"),
    OPERATOR_PATTERN      => _("Match Pattern")
    );

// array(
//   'type'      => QUERY_TYPE_...
//   'children'  => array(...) unless type == QUERY_TYPE_CRITERION
//   'criterion' => CRITERION_... if  type == QUERY_TYPE_CRITERION
//   'operator'  => OPERATOR_...  if  type == QUERY_TYPE_CRITERION
//   'value'     => other argument to operator of criterion
//   'ttype'     => associated ticket type of criterion

class Query {
    
    var $currentModule;
    var $currentTicketType;

    function Query()
    {
        global $backend;
        
        $modules = $backend->getModules();
        $module = array_shift(array_keys($modules));
        
        $types = $backend->getTypes($module);
        $ttype = array_shift(array_keys($types));
                             
        $this->setCurrentTicketType($module, $ttype);
        $this->reset();
    }

    // reset the query to the default empty query (or, at the moment, a slightly larger default query)
    function reset()
    {
        $cr1 = array(
            'type'      => QUERY_TYPE_CRITERION,
            'criterion' => CRITERION_ID,
            'operator'  => OPERATOR_EQUAL,
            'value'     => 1,
            'ttype'     => $this->currentTicketType);

        $cr2 = array(
            'type'      => QUERY_TYPE_CRITERION,
            'criterion' => CRITERION_ID,
            'operator'  => OPERATOR_EQUAL,
            'value'     => 5,
            'ttype'     => $this->currentTicketType);

        $cr3 = array(
            'type'      => QUERY_TYPE_CRITERION,
            'criterion' => CRITERION_ID,
            'operator'  => OPERATOR_EQUAL,
            'value'     => 7,
            'ttype'     => $this->currentTicketType);

        $orc = array($cr1, $cr2, $cr3);

        $or1 = array(
            'type'     => QUERY_TYPE_OR,
            'children' => $orc);

        $orc = array($cr1, $or1);

        $or2 = array(
            'type'     => QUERY_TYPE_OR,
            'children' => $orc);

        $cr4 = array(
            'type'      => QUERY_TYPE_CRITERION,
            'criterion' => CRITERION_REQUESTER,
            'operator'  => OPERATOR_PATTERN,
            'value'     => '%@hotmail.com');

        $andc = array($or1,/* $or2,*/ $cr4);

        $and = array(
            'type'     => QUERY_TYPE_AND,
            'children' => $andc);

        $this->query = $and; // array();
    }
    
    // properties
    function getCurrentModule()
    {
        return $this->currentModule;
    }

    function getCurrentTicketType()
    {
        return $this->currentTicketType;
    }

    function setCurrentTicketType($module, $type)
    {
        $this->currentModule = $module;
        $this->currentTicketType = $type;
    }

    // operations
    function delete($pathstring)
    {
        $path = Query::stringToPath($pathstring);
        $qobj = &$this->query;

        if (strlen($pathstring) == 0) {
            // deleting the root node is equivalent to creating a new blank query
            $this->reset();
        } else {
            $count = count($path) - 1;

            for ($i = 0; $i < $count; $i++) {
                $qobj = &$qobj['children'][$path[$i]];
            }

            array_splice($qobj['children'], $path[$count], 1);
        }
    }

    function hoist($pathstring) {
        $path = Query::stringToPath($pathstring);
        $qobj = &$this->query;

        if (strlen($pathstring) == 0) {
            // error! can't hoist the root node
        } else {
            $count = count($path) - 1;

            for ($i = 0; $i < $count; $i++) {
                $qobj = &$qobj['children'][$path[$i]];
            }

            $cobj = &$qobj['children'][$path[$count]];

            // TODO: make sure we're hoisting a branch
            array_splice($qobj['children'], $path[$count], 0, $cobj['children']);
            array_splice($cobj['children'], 0, count($cobj['children']));
        }
    }

    function insertBranch($pathstring, $type) {
        $path = Query::stringToPath($pathstring);
        $qobj = &$this->query;

        $newbranch = array(
            'type'     => $type,
            'children' => array());

        $count = count($path);

        for ($i = 0; $i < $count; $i++) {
            $qobj = &$qobj['children'][$path[$i]];
        }

        array_push($qobj['children'], $newbranch);
        array_push($path, count($qobj['children']) - 1);

        return Query::pathToString($path);
    }

    function insertCriterion($pathstring, $criterion, $operator, $value) {
        $path = Query::stringToPath($pathstring);
        $qobj = &$this->query;

        $newbranch = array(
            'type'      => QUERY_TYPE_CRITERION,
            'criterion' => $criterion,
            'operator'  => $operator,
            'value'     => $value,
            'ttype'     => $this->currentTicketType);

        $count = count($path);

        for ($i = 0; $i < $count; $i++) {
            $qobj = &$qobj['children'][$path[$i]];
        }

        array_push($qobj['children'], $newbranch);
    }

    // top down traversal
    function walk(&$obj, $method) {
        $path = array();
        $more = array();
        $this->_walk($this->query, $more, $path, $obj, $method);
    }

    function _walk(&$node, &$more, &$path, &$obj, $method) {
        if ($node['type'] == QUERY_TYPE_CRITERION) {
            $obj->$method($more, $path, QUERY_TYPE_CRITERION, $node['ttype'], $node['criterion'], $node['operator'], $node['value']);
        } else {
            $obj->$method($more, $path, $node['type'], null, null, null, null);
        }

        if (isset($node['children'])) {
            $count = count($node['children']);

            for ($i = 0; $i < $count; $i++) {
                array_push($path, $i);
                array_push($more, ($i < $count - 1));
                $this->_walk($node['children'][$i], $more, $path, $obj, $method);
                array_pop($more);
                array_pop($path);
            }
        }
    }

    // bottom up traversal
    function reduce(&$obj, $method) {
        return $this->_reduce($this->query, $obj, $method);
    }

    function _reduce(&$node, &$obj, $method)
    {
        $args = array();

        if (isset($node['children'])) {
            $count = count($node['children']);

            for ($i = 0; $i < $count; $i++) {
                $result = $this->_reduce($node['children'][$i], $obj, $method);
                array_push($args, $result);
            }
        }

        if ($node['type'] == QUERY_TYPE_CRITERION) {
            $result = $obj->$method($args, QUERY_TYPE_CRITERION, $node['ttype'], $node['criterion'], $node['operator'], $node['value']);
        } else {
            $result = $obj->$method($args, $node['type'], null, null, null, null);
        }

        return $result;
    }

    // saving / retrieving queries
    function save($name)
    {
        global $backend;

        $obj = serialize(array($this->currentModule, $this->currentTicketType, $this->query));

        $backend->saveQuery($name, $obj);
    }

    function load($name)
    {
        global $backend;

        $obj = &$backend->loadQuery($name);
        $obj = unserialize($obj);
        
        $this->currentModule = $obj[0];
        $this->currentTicketType = $obj[1];
        $this->query = $obj[2];
    }

    // static methods
    function pathToString(&$path) {
        return implode(',', $path);
    }

    function stringToPath($pathstring)
    {
        if (strlen($pathstring) == 0) {
            return array();
        }

        return explode(',', $pathstring);
    }

}

// Forms for editing queries
class InsertBranchForm extends Form {

    function InsertBranchForm(&$vars)
    {
        $this->Form($vars, _("Insert Branch"));

        $branchtypes = array(
            QUERY_TYPE_AND => _("And"),
            QUERY_TYPE_OR  => _("Or"),
            QUERY_TYPE_NOT => _("Not"));

        $branchtype = new EnumType($branchtypes);

        $var1 = new Variable(_("Path"), 'path', getTextType(), false, true);
        $var2 = new Variable(_("Branch Type"), 'type', $branchtype, true);

        $this->addHidden($var1);
        $this->addVariable($var2);
    }

}

class ChangeTypeForm extends Form {

    function ChangeTypeForm(&$vars)
    {
        global $backend;

        $this->Form($vars, _("Change Ticket Type"));

        $types = $backend->getAllTypes();
        $typestype = new EnumType($types);

        $var1 = new Variable(_("Current Type"), 'ttype', $typestype, true);

        $this->addVariable($var1);
    }

    // we want to render in a slightly modified form
    function getCustomRenderer()
    {
        return new QuerySetCurrentTypeRenderer();
    }

}

class UserCriterionForm extends Form {

    function UserCriterionForm(&$vars)
    {
        global $textOperators;
        global $backend;

        $this->Form($vars, _("Add User Criterion"));

        $optype = new EnumType($textOperators);

        $var1 = new Variable(_("Email Address"), 'email', getEmailType(), true);
        $var2 = new Variable(_("Match Operator"), 'operator', $optype, true);
        $var3 = new Variable(_("Search Assignee"), 'assignee', getBooleanType(), false);
        $var4 = new Variable(_("Search Requester"), 'requester', getBooleanType(), false);
        $var5 = new Variable(_("Search QA Contact"), 'qa', getBooleanType(), false);
        $var6 = new Variable(_("Search CC List"), 'cc', getBooleanType(), false);
        $var7 = new Variable(_("Search Comments"), 'comments', getBooleanType(), false);

        $this->addVariable($var1);
        $this->addVariable($var2);
        $this->addVariable($var3);
        $this->addVariable($var4);
        $this->addVariable($var5);
        $this->addVariable($var6);
        $this->addVariable($var7);
    }

    function execute(&$vars)
    {
        global $whups_query;
        global $backend;

        $path = $vars->getVar('path');
        $email = $vars->getVar('email');
        $operator = $vars->getVar('operator');
        $assignee = $vars->isVarSet('assignee');
        $requester = $vars->isVarSet('requester');
        $qa = $vars->isVarSet('qa');
        $cc = $vars->isVarSet('cc');
        $comments = $vars->isVarSet('comments');

        // if we're adding more than one criterion, put them all under an or node (which should be
        // what is wanted in the general case)
        if ($assignee + $requester + $qa + $cc + $comments > 1) {
            $path = $whups_query->insertBranch($path, QUERY_TYPE_OR);
        }

        if ($assignee) {
            $whups_query->insertCriterion($path, CRITERION_OWNER, $operator, $email);
        }

        if ($requester) {
            $whups_query->insertCriterion($path, CRITERION_REQUESTER, $operator, $email);
        }

        if ($qa) {
            $whups_query->insertCriterion($path, CRITERION_QA, $operator, $email);
        }

        if ($cc) {
            $whups_query->insertCriterion($path, CRITERION_CC, $operator, $email);
        }

        if ($comments) {
            $whups_query->insertCriterion($path, CRITERION_ADDED_COMMENT, $operator, $email);
        }

        $this->unsetVars($vars);
    }

}

class TextCriterionForm extends Form {

    function TextCriterionForm(&$vars)
    {
        global $textOperators;
        global $backend;

        $this->Form($vars, _("Add Text Criterion"));

        $optype = new EnumType($textOperators);

        $var1 = new Variable(_("Text"), 'text', getTextType(), true);
        $var2 = new Variable(_("Match Operator"), 'operator', $optype, true);
        $var3 = new Variable(_("Search Summary"), 'summary', getBooleanType(), false);
        $var4 = new Variable(_("Search Comments"), 'comments', getBooleanType(), false);

        $this->addVariable($var1);
        $this->addVariable($var2);
        $this->addVariable($var3);
        $this->addVariable($var4);
    }

    function execute(&$vars)
    {
        global $whups_query;
        global $backend;

        $path = $vars->getVar('path');
        $text = $vars->getVar('text');
        $operator = $vars->getVar('operator');
        $summary = $vars->isVarSet('summary');
        $comments = $vars->isVarSet('comments');

        if ($summary && $comments) {
            $path = $whups_query->insertBranch($path, QUERY_TYPE_OR);
        }

        if ($summary) {
            $whups_query->insertCriterion($path, CRITERION_SUMMARY, $operator, $text);
        }

        if ($comments) {
            $whups_query->insertCriterion($path, CRITERION_COMMENT, $operator, $text);
        }

        $this->unsetVars($vars);
    }

}

class PropertyCriterionForm extends Form {

    function PropertyCriterionForm(&$vars)
    {
        global $backend;

        $this->Form($vars, _("Add Property Criterion"));

        $type = $vars->getVar('ttype');

        $states = $backend->getStates($type);
        $states = array_merge(array('0' => _("Any")), $states);
        $states = new EnumType($states);

        $priorities = $backend->getPriorities($type);
        $priorities = array_merge(array('0' => _("Any")), $priorities);
        $priorities = new EnumType($priorities);

        $var1 = new Variable(_("Id"), 'id', getIntListType(), false);
        $var2 = new Variable(_("State"), 'state', $states, false);
        $var3 = new Variable(_("Priority"), 'priority', $priorities, false);

        $this->addVariable($var1);
        $this->addVariable($var2);
        $this->addVariable($var3);
    }

    function execute(&$vars)
    {
        global $whups_query;
        global $backend;

        $path = $vars->getVar('path');

        $id = $vars->getVar('id');
        if (trim($id) != '') {
            $newpath = $path;
            $ids = split("[\\t\\n ,]+", $id);

            if (count($ids) > 1) {
                $newpath = $whups_query->insertBranch($path, QUERY_TYPE_OR);
            }

            foreach ($ids as $id) {
                $whups_query->insertCriterion($newpath, CRITERION_ID, OPERATOR_EQUAL, $id);
            }
        }

        $state = $vars->getVar('state');
        if ($state > 0) {
            $whups_query->insertCriterion($newpath, CRITERION_STATE, OPERATOR_EQUAL, $state);
        }

        $priority = $vars->getVar('priority');
        if ($priority > 0) {
            $whups_query->insertCriterion($newpath, CRITERION_PRIORITY, OPERATOR_EQUAL, $priority);
        }

        $this->unsetVars($vars);
    }

}

class ChooseQueryNameForSaveForm extends Form {

    function ChooseQueryNameForSaveForm(&$vars)
    {
        global $backend;

        $this->Form($vars, _("Save Query"));

        $var1 = new Variable(_("Name"), 'name', getTextType(), true);

        $this->addVariable($var1);
    }

    function execute(&$vars)
    {
        global $whups_query;
        global $backend;

        $whups_query->save($vars->getVar('name'));

        $this->unsetVars($vars);
    }

}

class ChooseQueryNameForLoadForm extends Form {

    function ChooseQueryNameForLoadForm(&$vars)
    {
        global $backend;

        $this->Form($vars, _("Load Query"));

        $queries = $backend->getUserQueries(Auth::getAuth());
        $query_array = array();

        foreach ($queries as $var => $query) {
            $query_name = $query['query_name'];
            $query_array[$query_name] = $query_name;
        }

        $queries = new EnumType($query_array);

        $var1 = new Variable(_("Name"), 'name', $queries, true);

        $this->addVariable($var1);
    }

    function execute(&$vars)
    {
        global $whups_query;
        global $backend;

        $whups_query->load($vars->getVar('name'));

        $this->unsetVars($vars);
    }

}
?>
