<?php
/*
 * $Horde: whups/lib/Search.php,v 1.14 2002/02/20 18:05:27 chuck Exp $
 *
 * Copyright 2001 Robert E. Coyle <robertecoyle@hotmail.com>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/bsdl.php.
 */

require_once WHUPS_BASE . '/lib/Forms.php';

/**
 * SearchForm Class
 *
 * @author Robert E. Coyle <robertecoyle@hotmail.com>
 * @version $Revision: 1.14 $
 * @package whups
 */
class SearchForm extends Form {

    function SearchForm(&$vars)
    {
        global $backend;

        $this->Form($vars, '');

        $modules = $backend->getModules();
        $any = Whups::permissionsFilter(array('0' => _("Any")), 'module');
        $modules = array_merge($any, $modules);
        $modtype = new EnumType($modules);

        $var1 = new Variable(_("Ticket ID"), 'id', getIntListType(), false);
        $var2 = new Variable(_("Module Name"), 'module', $modtype, false);
        $var3 = new Variable(_("Count"), 'count', getIntType(), false, true);

        $var3->setDefault(10);

        $this->addVariable($var1);
        $this->addVariable($var2);
        $this->addHidden($var3);
    }

}

class SearchNarrow1Form extends Form {
    function SearchNarrow1Form(&$vars) {
	global $backend;

        $this->Form($vars, '');

        $type = $vars->getVar('type');

        $states = $backend->getStates($type);
        $states = array_merge(array('0' => _("Any")), $states);
        $states = new EnumType($states);

        $priorities = $backend->getPriorities($type);
        $priorities = array_merge(array('0' => _("Any")), $priorities);
        $priorities = new EnumType($priorities);

        $var1 = new Variable(_("Ticket State"), 'state', $states, false);
        $var2 = new Variable(_("Priority"), 'priority', $priorities, false);
        $var3 = new Variable(_("Hide Unconfirmed"), 'nouc', getBooleanType(), false);
        $var4 = new Variable(_("Hide New"), 'nonew', getBooleanType(), false);
        $var5 = new Variable(_("Hide Assigned"), 'noass', getBooleanType(), false);
        $var6 = new Variable(_("Hide Resolved"), 'nores', getBooleanType(), false);

        $var1->setDefault(0);
        $var2->setDefault(0);

        $this->addVariable($var1);
        $this->addVariable($var2);
        $this->addVariable($var3);
        $this->addVariable($var4);
        $this->addVariable($var5);
        $this->addVariable($var6);
    }
}

class TicketDetailsForm extends Form {

    function TicketDetailsForm(&$vars)
    {
        global $backend;

        $this->Form($vars);

        $var1 = new Variable(_("Ticket Summary"), 'summary', getTextType(), true, true);
        $var2 = new Variable(_("Module Name"), '_module_name', getTextType(), true, true);
        $var3 = new Variable(_("Ticket Type"), '_type_name', getTextType(), true, true);
        $var4 = new Variable(_("Ticket State"), '_state_name', getTextType(), true, true);
        $var5 = new Variable(_("Priority"), '_priority_name', getTextType(), true, true);
        $var6 = new Variable(_("Owner"), '_owner', getEmailType(), true, true);
        $var7 = new Variable(_("Requester"), '_requester', getEmailType(), true, true);

        $var6->setDefault(_("Unassigned"));

        $this->addVariable($var1);
        $this->addVariable($var2);
        $this->addVariable($var3);
        $this->addVariable($var4);
        $this->addVariable($var5);
        $this->addVariable($var6);
    }
}

class AddCommentForm extends Form {
    function AddCommentForm(&$vars) {
	global $backend;

        $this->Form($vars);

        $var1 = new Variable(_("Ticket ID"), 'id', getIntType(), true, true);
        $var2 = new Variable(_("Comment"), 'newcomment', getLongTextType(), true);

        $this->addHidden($var1);
        $this->addVariable($var2);
    }
}

class AssignTicketForm extends Form {
    function AssignTicketForm(&$vars) {
	global $backend;

        $this->Form($vars);

        $type = $vars->getVar('type');

        $states = $backend->getStates($type, 'assigned');
        $states = new EnumType($states);

        $module = $vars->getVar('module');
        $users = $backend->getModuleUsers($module);
        $usertype = new EnumType($users);

        $var1 = new Variable(_("Ticket Type"), 'type', getIntType(), true, true);
        $var2 = new Variable(_("Ticket ID"), 'id', getIntType(), true, true);
        $var3 = new Variable(_("Assignee"), 'assignee', $usertype, true);
        $var4 = new Variable(_("New State"), 'tostate', $states, true);
        $var5 = new Variable(_("Additional Comment"), 'newcomment', getLongTextType(), false);

        $this->addHidden($var1);
        $this->addHidden($var2);
        $this->addVariable($var3);
        $this->addVariable($var4);
        $this->addVariable($var5);
    }

    function getInfo(&$vars, &$info) {
        $this->_getInfo($vars, $info);
        $info['creator'] = Auth::getAuth();
    }
}

class SetStateForm extends Form {
    function SetStateForm(&$vars) {
	global $backend;

        $this->Form($vars);

        $type = $vars->getVar('type');
        $states = $backend->getStates($type, '', 'unconfirmed');
        $states = new EnumType($states);

        $var1 = new Variable(_("Ticket ID"), 'id', getIntType(), true, true);
        $var2 = new Variable(_("Type"), 'type', getIntType(), true, true);
        $var3 = new Variable(_("New State"), 'state', $states, true);
        $var4 = new Variable(_("Comment"), 'newcomment', getLongTextType(), true);

        $this->addHidden($var1);
        $this->addHidden($var2);
        $this->addVariable($var3);
        $this->addVariable($var4);
    }
}

class SetPriorityForm extends Form {
    function SetPriorityForm(&$vars) {
	global $backend;

        $this->Form($vars);

        $type = $vars->getVar('type');
        $priorities = $backend->getPriorities($type);
        $priorities = new EnumType($priorities);

        $var1 = new Variable(_("Ticket ID"), 'id', getIntType(), true, true);
        $var2 = new Variable(_("Type"), 'type', getIntType(), true, true);
        $var3 = new Variable(_("New Priority"), 'priority', $priorities, true);
        $var4 = new Variable(_("Comment"), 'newcomment', getLongTextType(), true);

        $this->addHidden($var1);
        $this->addHidden($var2);
        $this->addVariable($var3);
        $this->addVariable($var4);
    }
}

?>
