<?php
/*
 * $Horde: whups/lib/WhupsVars.php,v 1.6 2002/02/20 18:06:07 chuck Exp $
 *
 * Copyright 2001 Robert E. Coyle <robertecoyle@hotmail.com>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/bsdl.php.
 */

/**
 * WhupsVars Class
 *
 * @author Robert E. Coyle <robertecoyle@hotmail.com>
 * @version $Revision: 1.6 $
 * @package whups
 */
class WhupsVars {
    function WhupsVars(&$vars, &$parent) {
        $this->vars   = $vars;
        $this->parent = $parent;
        $this->prefix = array('');
    }

    function unsetVar($varname) {
        $this->_unsetVar($this->prefix[0] . $varname);
    }

    function getVar($varname) {
        $result = $this->_getVar($this->prefix[0] . $varname, $ignore);
        return $result;
    }

    function addVar($varname, $value) {
        if ($this->getVar($varname)) {
            return false;
        }
        $this->vars[$varname] = $value;
    }

    function getVarWasset($varname, &$wasset) {
        $result = $this->_getVar($this->prefix[0] . $varname, $wasset);
        return $result;
    }

    function isVarSet($varname) {
        $wasset = false;
        $this->getVarWasset($varname, $wasset);
        return $wasset;
    }

    function getPrefix() {
        return $this->prefix[0];
    }

    function pushPrefix($prefix) {
        array_unshift($this->prefix, $prefix);
    }

    function popPrefix() {
        array_shift($this->prefix);
    }

    function preserveByGet($baseurl) {
        $this->_preserveByGet($baseurl, $this->prefix[0]);
    }

    function preserveByPost() {
        $this->_preserveByPost($this->prefix[0]);
    }

    function preserveVarByPost($varname) {
        $varname = $this->prefix[0] . $varname;
        $value   = $this->_getVar($varname, $wasset);

        if ($wasset)
            preserveVarByPost($varname, $value);
    }

    // -- private functions --

    function _unsetVar($varname) {
        if (is_object($this->parent))
            $this->parent->_unsetVar($varname);

        unset($this->vars[$varname]);
    }

    function _getVar($varname, &$wasset) {
        if (is_object($this->parent)) {
            $var = $this->parent->_getVar($varname, $wasset);

            if ($wasset) {
                return $var;
            }
        }

        $wasset = isset($this->vars[$varname]);
        return ($wasset ? Horde::dispelMagicQuotes($this->vars[$varname]) : null);
    }

    function _preserveByGet($baseurl, $prefix) {
        if ($this->parent) {
            $baseurl = $this->parent->_preserveByGet($baseurl, $prefix);
        }

        if (strchr($baseurl, '?')) {
            $join = '&';
        } else {
            $join = '?';
        }

        $prefixlen = strlen($prefix);

        foreach ($this->vars as $name => $value) {
            if (substr($name, 0, $prefixlen) == $prefix) {
                $baseurl .= $join . urlencode($name) . '=' . urlencode($value);
                $join = '&';
            }
        }

        return $baseurl;
    }

    function _preserveByPost($prefix) {
        if ($this->parent) {
            $this->parent->_preserveByPost($prefix);
        }

        $prefixlen = strlen($prefix);

        foreach ($this->vars as $name => $value) {
            if (substr($name, 0, $prefixlen) == $prefix) {
                preserveVarByPost($name, $value);
            }
        }
    }
}

function &getDefaultVars($override = null) {
    global $HTTP_GET_VARS;
    global $HTTP_POST_VARS;

    return new WhupsVars($HTTP_POST_VARS,
                         new WhupsVars($HTTP_GET_VARS, $override));
}

?>
