<?php
/*
 * $Horde: whups/query.php,v 1.6 2002/01/25 19:37:24 chuck Exp $
 *
 * Copyright 2001 Robert E. Coyle <robertecoyle@hotmail.com>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/bsdl.php.
 */

define('WHUPS_BASE', dirname(__FILE__));
require_once WHUPS_BASE . '/lib/base.php';
require_once WHUPS_BASE . '/lib/Search.php';

require WHUPS_TEMPLATES . '/common.inc';
require WHUPS_TEMPLATES . '/page.inc';
require WHUPS_TEMPLATES . '/headerbar.inc';
require WHUPS_TEMPLATES . '/actions.inc';
require WHUPS_TEMPLATES . '/formrenderer.inc';
require WHUPS_TEMPLATES . '/queryrenderer.inc';
require WHUPS_TEMPLATES . '/searchresults.inc';
require WHUPS_TEMPLATES . '/useractions.inc';

useForms(array(
    'InsertBranchForm',
    'ChangeTypeForm',
    'UserCriterionForm',
    'TextCriterionForm',
    'PropertyCriterionForm',
    'SearchNarrow1Form',
    'ChooseQueryNameForSaveForm',
    'ChooseQueryNameForLoadForm'));

// set up the page config vars
$query_action  = '';
$showTypeForm  = true;
$showEditQuery = true;
$showExtraForm = null;

// query actions
$QACTIONS = new Actions();
$QACTIONS->addAction(_("New Query"), 'query.php', 'new');
$QACTIONS->addAction(_("Execute Query"), 'query.php', 'execute');
$QACTIONS->addAction(_("Load Query"), 'query.php', 'load');
$QACTIONS->addAction(_("Save Query"), 'query.php', 'save');

// criterion form types
$CACTIONS = new Actions();
$CACTIONS->addAction(_("Property Criteria"), 'query.php', 'props');
$CACTIONS->addAction(_("User Criteria"), 'query.php', 'user');
$CACTIONS->addAction(_("Date Criteria"), 'query.php', 'date');
$CACTIONS->addAction(_("Text Criteria"), 'query.php', 'text');

$PAGE = new Page();
$HEADERBAR = new HeaderBar();
$RENDERER = new FormRenderer();
$QUERYRENDERER = new QueryRenderer();

$qops = array(
    ''       => _("Choose Action:"),
    'delete' => _("Delete"),
    'edit'   => _("Edit"),
    'move'   => _("Move"),
    'hoist'  => _("Hoist"),
    'and'    => _("Insert And"),
    'or'     => _("Insert Or"),
    'not'    => _("Insert Not"),
    'query'  => _("Insert Query"));

$beendone = 0;

$vars = &getDefaultVars();
$info = array();

// find our current query
//if (!isset($HTTP_SESSION_VARS['whups_query'])) {
    $HTTP_SESSION_VARS['whups_query'] = new Query();
//}

$whups_query = &$HTTP_SESSION_VARS['whups_query'];
session_register('whups_query');

// find the current criteria form, and default to the user form if not present
if (!isset($HTTP_SESSION_VARS['whups_query_form'])) {
    $HTTP_SESSION_VARS['whups_query_form'] = 'user';
}

$whups_query_form = $HTTP_SESSION_VARS['whups_query_form'];
session_register('whups_query_form');

// setup vars with all relevant info
$ttype = array('ttype' => $whups_query->getCurrentTicketType());
$vars = new WhupsVars($ttype, $vars);

// whatnow?
// first check the result of the query edit action dropdown, as this action
// overrides the form it sits within
if ($vars->getVar('qaction') != '') {
    $action = $vars->getVar('qaction');

    switch ($action) {
    // Query actions
    case 'delete': $whups_query->delete($vars->getVar('path')); $vars->unsetVar('path'); break;
    case 'hoist':  $whups_query->hoist($vars->getVar('path')); break;
    case 'branch': $form = &getInsertBranchForm($vars); break;
    case 'not':    $whups_query->insertBranch($vars->getVar('path'), QUERY_TYPE_NOT); break;
    case 'and':    $whups_query->insertBranch($vars->getVar('path'), QUERY_TYPE_AND); break;
    case 'or':     $whups_query->insertBranch($vars->getVar('path'), QUERY_TYPE_OR); break;
    case 'edit':
    case 'move':
    case 'query':  debug(_("Not yet implemented..."));
    }

    $vars->unsetVar('qaction');
} else if ($vars->isVarSet('formname')) {
    // now check for submitted forms
    $form = &getForm($vars->getVar('formname'), $vars);
    $form->validate($vars);

    // TODO: $form->execute should result in a call to ACTIONRESULT->result()
    if ($form->isValid()) {
        $form->execute($vars);
        $vars->unsetVar('action');
    }
} else if ($vars->getVar('action') != '') {
    // lastly, check for actions from the useractions template class
    $action = $vars->getVar('action');

    switch ($action) {
        // Current form actions
    case 'props':
    case 'user':
    case 'date':
    case 'text':
        $whups_query_form = $action;
        break;

        // Global query options
    case 'new':
        $whups_query = new Query();
        // $whups_query->reset();
        break;

    case 'execute':
        // TODO: Actually execute...
        $backend->executeQuery($whups_query);
        break;

    case 'save':
        $showExtraForm  = 'ChooseQueryNameForSaveForm';
        $showTypeForm   = false;
        $showEditQuery  = false;
        break;

    case 'load':
        $showExtraForm  = 'ChooseQueryNameForLoadForm';
        $showTypeForm   = false;
        $showEditQuery  = false;
        break;
    }
}

// start the page
$PAGE->open(_("Query Builder"));
$HEADERBAR->header($sections, 'query');
$HEADERBAR->actions($sections, 'query');

// TODO: ACTIONRESULT->result of above operations

if ($showExtraForm !== null) {
    $form = &getForm($showExtraForm, $vars);
    $form->open($RENDERER, $vars, 'query.php');
    $RENDERER->beginActive($form->getTitle());
    $RENDERER->renderFormActive($form, $vars);
    $RENDERER->submit();
    $RENDERER->end();
    $form->close($RENDERER);    
}

if ($showTypeForm) {
    $typeform = &getChangeTypeForm($vars);
    $typeform->open($RENDERER, $vars, 'query.php');
    $RENDERER->beginActive($typeform->getTitle());
    $RENDERER->renderFormActive($typeform, $vars);
    $RENDERER->end();
    $typeform->close($RENDERER);
}

if ($showEditQuery) {
    // Get our current form
    switch ($whups_query_form) {
    default:
        debug(sprintf(_("Error: Unknown query form %s, defaulting to properties"), $whups_query_form));
        // fallthrough

    case 'props': $form = &getPropertyCriterionForm($vars); break;
    case 'user': $form = &getUserCriterionForm($vars); break;
    case 'text': $form = &getTextCriterionForm($vars); break;
    }

    $form->open($RENDERER, $vars, 'query.php', 'post');

    $RENDERER->beginActive(_("Current Query"));
    $QUERYRENDERER->edit($qops, $form->getName());
    $QUERYRENDERER->renderActive($whups_query, $vars);
    $RENDERER->end();

    $CACTIONS->render($whups_query_form);

    $RENDERER->beginActive($form->getTitle());
    $RENDERER->renderFormActive($form, $vars);
    $RENDERER->submit(_("Add Criterion"));
    $RENDERER->end();

    $form->close($RENDERER);
} else {
    // show query readonly
    $RENDERER->beginActive(_("Current Query"));
    $QUERYRENDERER->renderInactive($whups_query, $vars);
    $RENDERER->end();
}

$QACTIONS->render($query_action);

$PAGE->close();

?>
