--
-- $Horde: whups/scripts/drivers/whups.sql,v 1.9 2002/02/28 04:08:54 chuck Exp $
--
-- Copywrite 2001 Robert E. Coyle <robertecoyle@hotmail.com>
--
-- See the enclosed file LICENSE for license information (BSD). If you
-- did not receive this file, see http://www.horde.org/bsdl.php.
--
-- Database definitions for Whups

CREATE TABLE whups_tickets (
    ticket_id           INT NOT NULL,          -- unique ticket id
    ticket_summary      VARCHAR(255),          -- summary of the ticket
    user_id_requester   VARCHAR(255) NOT NULL, -- user id of the creator of this ticket
    user_id_owner       VARCHAR(255) NOT NULL, -- user id of the currently assigned owner
    module_id           INT NOT NULL,          -- module id that this ticket referres to
    type_id             INT NOT NULL,          -- id into the type table, describing the type of ticket
    state_id            INT NOT NULL,          -- state of this ticket, meaning depends on the type of the ticket
    priority_id         INT NOT NULL,          -- priority, meaning depends on the type of the ticket
    comment_id          INT NOT NULL,          -- id in the comment table - describes the ticket
    ticket_timestamp    INT NOT NULL,          -- redundant but useful, mirrored in the comment and state_log

    PRIMARY KEY (ticket_id)
);

CREATE TABLE whups_modules (
    module_id           INT NOT NULL,
    module_name         VARCHAR(64) NOT NULL,
    module_description  VARCHAR(255),
    module_subjectlist  SMALLINT DEFAULT 0 NOT NULL,

    PRIMARY KEY (module_id)
);

CREATE TABLE whups_modules_users (
    module_id INT NOT NULL,
    user_uid  VARCHAR(250) NOT NULL,

    PRIMARY KEY (module_id, user_uid)
);

CREATE TABLE whups_subjects (
    subject_id    INT NOT NULL,
    subject_name  VARCHAR(255),

    PRIMARY KEY (subject_id)
);

CREATE TABLE whups_modules_subjects (
    module_id INT NOT NULL,
    subject_id INT NOT NULL,

    PRIMARY KEY (module_id, subject_id)
);

CREATE TABLE whups_types (
    type_id           INT NOT NULL,
    type_name         VARCHAR(64) NOT NULL,
    type_description  VARCHAR(255),
 
    PRIMARY KEY (type_id)
);

CREATE TABLE whups_types_modules (
    type_id           INT NOT NULL,
    module_id         INT NOT NULL
);

CREATE TABLE whups_states (
    state_id           INT NOT NULL,
    type_id            INT NOT NULL,
    state_name         VARCHAR(64) NOT NULL,
    state_description  VARCHAR(255),
    state_category     VARCHAR(16),

    PRIMARY KEY (state_id)
);

CREATE TABLE whups_comments (
    comment_id           INT NOT NULL,
    ticket_id            INT NOT NULL,
    user_id_creator      VARCHAR(255) NOT NULL,
    comment_text         TEXT,
    comment_timestamp    INT,

    PRIMARY KEY (comment_id)
);

CREATE TABLE whups_state_logs (
    ticket_id          INT NOT NULL,
    state_from         INT,
    state_to           INT NOT NULL,
    state_timestamp    INT NOT NULL,
    user_id_owner      VARCHAR(255) NOT NULL,
    state_explanation  INT NOT NULL
);

CREATE TABLE whups_attachments (
    attachment_id           INT NOT NULL,
    attachment_name         VARCHAR(255) NOT NULL,
    comment_id              INT NOT NULL,
    attachment_mime_type    VARCHAR(255) NOT NULL,
    attachment_body         TEXT NOT NULL,

    PRIMARY KEY (attachment_id)
);

CREATE TABLE whups_priorities (
    priority_id           INT NOT NULL,
    type_id               INT NOT NULL,
    priority_name         VARCHAR(64),
    priority_description  VARCHAR(255),

    PRIMARY KEY (priority_id)
);

CREATE TABLE whups_users_searches (
    user_id             VARCHAR(255) NOT NULL,
    search_name         VARCHAR(255),
    search_getvars      VARCHAR(255) -- FIXME: big enough?
);

CREATE TABLE whups_users_queries (
    user_id             VARCHAR(255) NOT NULL,
    query_name          VARCHAR(255),
    query_object        TEXT
);

CREATE TABLE whups_tickets_listeners (
    ticket_id       INT NOT NULL,
    user_id         VARCHAR(255) NOT NULL
);
