<?php
/*
 * $Horde: whups/templates/formrenderer.inc,v 1.13 2002/03/02 19:16:41 chuck Exp $
 *
 * Copyright 2001 Robert E. Coyle <robertecoyle@hotmail.com>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/bsdl.php.
 */

class FormRenderer {
    function open($action, $method, $name) {
        echo "<form action=\"$action\" method=\"$method\"" . (empty($name) ? "" : " name=\"$name\"") . ">\n";
        Horde::pformInput();
    }

    function beginActive($name) {
        $this->_renderBeginActive($name);
    }

    function beginInactive($name) {
        $this->_renderBeginInactive($name);
    }

    function end() {
        $this->_renderEnd();
    }

    function close() {
        echo "</form>\n";
    }

    function renderFormActive(&$form, &$vars) {
        $this->renderForm($form, $vars, true);	
    }

    function renderFormInactive(&$form, &$vars) {
        $this->renderForm($form, $vars, false);
    }

    function renderForm(&$form, &$vars, $active) {
        // check for a custom renderer for this form
        $customRenderer = $form->getCustomRenderer();

        if ($customRenderer !== null) {
            return $customRenderer->render($form, $vars, $active);
        }

        // no custom form - we'll do it our way
        $variables = &$form->getVariables();

        foreach ($variables as $var) {
            $type = $var->getTypeName();

            $format = (!$active || $var->isReadonly()) ? 'Display' : 'Input';
            $begin = "_renderVar${format}Begin";
            $end = "_renderVar${format}End";
            $method = "_renderVar${format}_$type";

            $this->$begin($form, $var, $vars);

            if (@method_exists($this, $method) == false) {
                $method = "_renderVar${format}Default";
            }
        
            $this->$method($form, $var, $vars);
            $this->$end($form, $var, $vars);
        }
    }

    function submit($submit = null) {
        if (!isset($submit)) {
            $submit = _("Submit");
        }
        $this->_renderSubmit($submit);
    }

    // Implementation specifics -- begin / end functions

    function _renderBeginActive($name) {
	sectionHeader($name);
?>
<table class="form" border="0" width="100%" cellspacing="1" cellpadding="2">
<?php
    }

    function _renderBeginInactive($name) {
	sectionHeader($name);
?>
<table class="form" border="0" width="100%" cellspacing="1" cellpadding="2">
<?php
    }

    function _renderEnd() {
?>
</table>

<br />

<?php
    }

    function _renderSubmit($submit) {
?>
<tr><td colspan="2" class="item0"><input class="button" name="submit" type="submit" value="<?= $submit ?>" /></td></tr>
<?php
    }

    // Implementation specifics -- input variables

    function _renderVarInputBegin(&$form, &$var, &$vars) {
	$message = $form->getError($var);
	$isvalid = empty($message);

	$req  = $var->isRequired() ? '* ' : '';
	$pre  = $isvalid ? '' : '<span class="form-error">';
	$post = $isvalid ? '' : ('<br />' . ucfirst($message) . '</span>');
?>
<tr class="item0">
 <td width="25%"><?= $pre . $req . $var->getHumanName() . $post ?></td>
 <td width="75%">
<?php
    }

    function _renderVarInputEnd(&$form, &$var, &$vars) {
?>
 </td>
</tr>
<?php
    }

    function _renderVarInputDefault(&$form, &$var, &$vars) {
?>
<strong>Warning:</strong> Unknown variable type <?= $var->getTypeName() ?>
<?php
    }

    function _renderVarInput_int(&$form, &$var, &$vars) {
?>
<input type="text" name="<?= $var->getVarName() ?>" value="<?= $var->getValue($vars) ?>" />
<?php
    }

    function _renderVarInput_intlist(&$form, &$var, &$vars) {
?>
<input type="text" name="<?= $var->getVarName() ?>" value="<?= $var->getValue($vars) ?>" />
<?php
    }

    function _renderVarInput_text(&$form, &$var, &$vars) {
?>
<input type="text" name="<?= $var->getVarName() ?>" value="<?= $var->getValue($vars) ?>" />
<?php
    }

    function _renderVarInput_longtext(&$form, &$var, &$vars) {
?>
<textarea name="<?= $var->getVarName() ?>" cols="80" rows="8"><?= $var->getValue($vars) ?></textarea>
<?php
    }

    function _renderVarInput_invalid(&$form, &$var, &$vars) {
        return $this->_renderVarDisplay_invalid($form, $var, $vars);
    }

    function _renderVarInput_enum(&$form, &$var, &$vars) {
        $values = $var->getValues();
        $selected  = $var->getValue($vars);
?>
<select name="<?= $var->getVarName() ?>">
<?= generateSelectOptions($values, $selected) ?>
</select>
<?php
    }

    function _renderVarInput_set(&$form, &$var, &$vars) {
        $values = $var->getValues();
        $value  = $var->getValue($vars);
        echo generateCheckBoxes($var->getVarName(), $values, $value);
    }

    function _renderVarInput_link(&$form, &$var, &$vars)
    {
        return _renderVarDisplay_link($form, $var, $vars);
    }

    function _renderVarInput_user(&$form, &$var, &$vars) {
?>
<input type="text" name="<?= $var->getVarName() ?>" value="<?= $var->getValue($vars) ?>" size="35" />
<?php
    }

    function _renderVarInput_email(&$form, &$var, &$vars) {
?>
<input type="text" name="<?= $var->getVarName() ?>" value="<?= $var->getValue($vars) ?>" />
<?php
    }

    function _renderVarInput_boolean(&$form, &$var, &$vars) {
?>
<input type="checkbox" name="<?= $var->getVarName() ?>"<?= $var->getValue($vars) ? ' checked="checked"' : '' ?> />
<?php
    }

    // Implementation specifics -- display variables

    function _renderVarDisplayBegin(&$form, &$var, &$vars) {
?>
<tr class="item0">
 <td width="25%"><?= $var->getHumanName() ?></td>
 <td width="75%">
<?php
    }

    function _renderVarDisplayEnd(&$form, &$var, &$vars) {
?>
 </td>
</tr>
<?php
    }

    function _renderVarDisplayDefault(&$form, &$var, &$vars) {
        echo htmlspecialchars($var->getValue($vars));
    }

    function _renderVarDisplay_email(&$form, &$var, &$vars) {
        $value = $var->getValue($vars);
        $id = $var->getVarName();

        if (strpos($value, '@') === false) {
            echo $value;
        } else {
            // see if we've got a name for the address
            $name = $vars->getVar($id . '_name');

            $value = urlencode($value);
            $name = htmlspecialchars($name);

            if (!empty($name)) {
                echo "<a href=\"mailto:$value\">$name</a>";
            } else {
                echo "<a href=\"mailto:$value\">$value</a>";
            }
        }
    }

    function _renderVarDisplay_boolean(&$form, &$var, &$vars)
    {
        echo $var->getValue($vars) == 'on' ? _("On") : _("Off");
    }

    function _renderVarDisplay_enum(&$form, &$var, &$vars) {
        $values = $var->getValues();
        if (count($values) > 0) {
            echo htmlspecialchars($values[$var->getValue($vars)]);
        }
    }

    function _renderVarDisplay_set(&$form, &$var, &$vars)
    {
        $values = $var->getValues();
        $on = $var->getValue($vars);
        if (count($values) > 0 && count($on) > 0) {
            $display = array();
            foreach ($values as $value => $name) {
                if (in_array($value, $on)) {
                    $display[] = $name;
                }
            }
            echo htmlspecialchars(implode(', ', $display));
        }
    }

    function _renderVarDisplay_invalid(&$form, &$var, &$vars)
    {
        echo '<span class="form-error">' . htmlspecialchars($var->type->message) . '</span>';
    }

    function _renderVarDisplay_link(&$form, &$var, &$vars)
    {
        if (empty($var->type->values['url']) || empty($var->type->values['text'])) {
            return;
        }
        echo Horde::link($var->type->values['url'], $var->type->values['text'], 'widget') . $var->type->values['text'] . '</a>';
    }

}

?>
